function options = CheckOptions( options )

    %this method checks, if all required field are set in options; in case
    %of missing fields default values are used or an error is thrown
    
    %check required fields
    RequiredFields  = {'Gmax','Smax','Vmax',...
                       'b1maps','b0map','roi','targetflip_mat','xpos_vec','ypos_vec','zpos_vec',...
                       'trajectory_fun','phi_init','phi_bounds'};
    MssingFields    = RequiredFields( cellfun( @( str ) ~isfield( options, str ), RequiredFields ) );

    if ~isempty( MssingFields )
        error(['Missing required fields in ''options'':',sprintf('\n - %s',MssingFields{:})])
    end
        
    %check size consistency of maps
    [ nx, ny, nz, ~ ] = size( options.b1maps );
    
    if any( [nx,ny,nz] ~= size( options.b0map ) )
        error('Size of b0map does not match size of b1maps!')
    end
    if any( [nx,ny,nz] ~= size( options.roi ) )
        error('Size of roi does not match size of b1maps!')
    end
    if any( [nx,ny,nz] ~= size( options.targetflip_mat ) )
        error('Size of targetflip_mat does not match size of b1maps!')
    end
    
    if any( nx ~= length( options.xpos_vec ) )
        error('Size of X_vec does not match size of b1maps!')
    end
    if any( ny ~= length( options.ypos_vec ) )
        error('Size of Y_vec does not match size of b1maps!')
    end
    if any( nz ~= length( options.zpos_vec ) )
        error('Size of Z_vec does not match size of B1_mat!')
    end
        
    %check size consistency of shape parameters
    nshapeparams = numel( options.phi_init );
    
    %check sizes of shape parameter option vectors
    if all( size( options.phi_init ) == [1, nshapeparams] )
        %convert to column vector
        options.phi_init = options.phi_init.';
    elseif any( size( options.phi_init ) ~= [nshapeparams, 1] )
        error('Phi_init must be a vector!')
    end
    
    if all( size( options.phi_bounds ) == [2, nshapeparams] )
        %convert to nx2 matrix
        options.phi_bounds = options.phi_bounds.';
    elseif any( size( options.phi_bounds ) ~= [nshapeparams, 2] )
        error('Phi_bounds must be a %ix2 matrix!',nshapeparams,nshapeparams)
    end
    
    if ~isfield( options, 'phi_isvariable' )
        %all shape parameters are being optimized by default
        options.phi_isvariable = ones(size(options.phi_init));
    elseif all( size( options.phi_isvariable ) == [1, nshapeparams] )
        %convert to nx2 matrix
        options.phi_isvariable = options.phi_isvariable.';
    elseif any( size( options.phi_isvariable ) ~= [nshapeparams, 1] )
        error('Phi_isvariable must be a vector of length %i!',nshapeparams)
    end
    
    %check if initial vector conforms bounds
    if      any( options.phi_init < options.phi_bounds(:,1) | options.phi_init > options.phi_bounds(:,2) )
        error('Initial shape parameters violate the given shape parameter bounds!') 
    end
    
    %  B 0 ,   B 1 ,   R O I ,   T A R G E T
    %----------------------------------------
    % integer number of undersampling in x/y/z, field maps will be averaged
    options = SetDefaultField( options, 'undersam_x', 1 );
    options = SetDefaultField( options, 'undersam_y', 1 );
    options = SetDefaultField( options, 'undersam_z', 1 );
    % blur target pattern to remove energy in tranismit k-space at high k
    options = SetDefaultField( options, 'blur_target', 0 );
    
    %  S Y S T E M   M A T R I X
    %----------------------------
    % prevent the solver from allocating excessively large system matrices
    % which may lead to a crash of Matlab
    options = SetDefaultField( options, 'sysmat_maxsize', 8.5e+9 );
    % number of interpolation steps between two consecutive k-space
    % samples, when setting up the system matrix
    options = SetDefaultField( options, 'sysmat_interp', 10 );
    % use GPU, e.g., for multiplication of A' * A for the conjugate
    % gradient RF solver
    options = SetDefaultField( options, 'use_gpu', false );
    % this value specifies the offset frequency in Hz, for which the RF is
    % also optimized additionally to the optimization at Larmor frequency
    % w. I.e., for a value of 50Hz, the RF pulse will be optimized at 
    % { w-50Hz, w, w+50Hz }. This option significantly increases
    % computation time.
    options = SetDefaultField( options, 'enforceB0', 0 );
    
    %  R F   A N D   G R A D I E N T S
    %----------------------------------
    % sampling time of gradient and RF waveforms. It is also very easy to
    % implement different sampling times for the gradient and RF waveforms
    options = SetDefaultField( options, 'grad_dt', 10e-6 ); 
    % initial guess of the RF peak power regularization parameter. The
    % newton based method for search of the regularization parameter
    % satisfying Vmax will be initiated with this value.
    options = SetDefaultField( options, 'alpha_power', -3.5 ); 
    % set to true, if the regularization parameter should be updated after
    % each iteration. This may work in certain cases, but has the
    % disadvantage, that the objective function changes during runtime
    % without being noticed by the optimizer (i.e., fmincon), which may
    % produce suboptimial results in certain cases. If during the
    % optimization you notice that Vmax is greatly violated (i.e., alpha 
    % becomes widely over- or undererstimated), you may want to use the
    % constrained least squares RF solver instead (optim_rfsolver = 2)
    options = SetDefaultField( options, 'alpha_update', false ); 
    % shift gradient and RF waveforms against each other by RF_delay
    % seconds (this may be needed for fine tuning of the RF pulses)
    options = SetDefaultField( options, 'RF_delay', 0 ); 
    % maximum allowed RF duration for which and RF pulse will be computed.
    % Otherwise a function value of Inf will be assumed. This significantly
    % reduces the computation time
    options = SetDefaultField( options, 'pulse_maxtime', 0.08 );
    % run bloch simulations for initial and optimized RF pulse and store
    % the results
    options = SetDefaultField( options, 'run_blochsim', true );
    % number of interpolation steps between two consecutive k-space
    % samples, when performing the bloch simulation
    options = SetDefaultField( options, 'blochsim_interp', 10 );
	
    % O B J E C T I V E   F U N C T I O N
    %-------------------------------------
    % regularization strength of the RF pulse duration. The default value
    % of 2.0 means, that a pulse duration of 1ms above the limit Tmax is
    % acceptable if, at the same time, the NRMSE can be reduced by 2.0%.
    % I.e., increasing beta will enforce the anticipated pulse duration 
    % given by Tmax. Note that pulse durations less than Tmax are not
    % penalized at all, as illustrated in our publication.
    options = SetDefaultField( options, 'objfun_beta', 2.0 );
    % Anticipated pulse duration in seconds
    options = SetDefaultField( options, 'objfun_Tmax', 0.005 );
    % Transition width between non-penalized (T < Tmax) and penalized (T > Tmax)
    % durations
    options = SetDefaultField( options, 'objfun_epsilon', 0.0015 );

    % O P T I M I Z A T I O N
    %-------------------------
    % Type of RF solver: 
    % 1: Tikhonov Least Squares (Conjugate Gradient solver)
    % 2: Constrained Least Squares (fmincon with constraints)
    options = SetDefaultField( options, 'optim_rfsolver', 1 );
    % Type of finite differencing used within the optimization, this should
    % not be changed
    options = SetDefaultField( options, 'optim_findifftype', 2 );
    % Normalize optimization variables during optimization, this should not 
    % be changed
    options = SetDefaultField( options, 'optim_scalevariables', 1 );
    % Termination criteria for fmincon
    options = SetDefaultField( options, 'optim_tolfun', 1e-3 );
    options = SetDefaultField( options, 'optim_tolcon', 1e-5 );
    options = SetDefaultField( options, 'optim_tolx', 1e-5 );
    % Maximum number of iterations and function evaluations
    options = SetDefaultField( options, 'optim_maxiter', 50 );
    options = SetDefaultField( options, 'optim_maxfunevals', +Inf );
    % Select how to proceed, if an error occures during the objective
    % function evaluation
    % 1: return objective function value of Inf, continue
    % 2: throw error and terminate optimization
    options = SetDefaultField( options, 'optim_errhandling', 1 );
    % Saves figure after each iteration, showing the current gradient
    % trajectory, flip angle maps, gradient and RF waveforms
    options = SetDefaultField( options, 'optim_saveiterfig', 1 );
    % Option for debuging, produces some more figures and does not save any
    % results, this is useful for tuning the initial gradient trajectory
    options = SetDefaultField( options, 'optim_debugmode', 0 );
    
    % Options struct for the Time-Optimal Gradient Design Method, these
    % options produce very reliable results and should only be changed
    % carefully
    options = SetDefaultField( options, 'GBF', struct(  'OnlyContinuousRanges',0,...
                                                        'GradSmooth_On',1,...
                                                        'GradSmooth_Lambda',1e-3,...
                                                        'GradSmooth_MaxDistFac',2,...
                                                        'verbosity',0,...
                                                        'UseMex',1,...
                                                        'SolverID','MinConF_SPG' ) );

end

function options = SetDefaultField( options, fieldname, defaultvalue )

    if ~isfield( options, fieldname )
        options.(char(fieldname)) = defaultvalue;
    end
end