function [ RF_final, Err_final ] = DesignPulse_ConjugateGradient( options, C, d, RF_0, MaxWhileIter )

    % This method solves the RF design problem based on a Tikhonov least
    % squares formalism. This method is faster than the constraint LS
    % solver in DesignPulse_LeastSquares.m but designs less powerful
    % RF pulses and may not respect the RF peak power constraint at all
    % samples

    global cur_alpha_power

    %default accuracy
    Err_final       = nan;
    tol             = 1e-6;
    MaxCGIter       = 10000;
    
    ncunk   = size( C, 2 ); %--> NumChannels * NumberKtPoints
    
    if nargin < 4
        RF_0 	= zeros(ncunk, 1);
    else
        if isempty( RF_0 )
            RF_0 	= zeros(ncunk, 1);
        end
    end
    
    if nargin < 5 || isempty( MaxWhileIter )
     	MaxWhileIter  	= 1;
    end
           
    if MaxWhileIter > 1
        IterOutput      = 1;
    else
        IterOutput      = 0;
    end
        
    %initial regularization constant
    Iter                = 1;
    RF_final            = nan;
    alpha_powerfindiff  = 0.2;
    alpha_powermaxstep  = 1.0;
    PeakRFtol           = 0.1 * options.Vmax;
    
    %prepare a cell to store the RF pulses, max 2*MaxWhileIter pulses
    RF_pulses_cell = cell(2*MaxWhileIter,3);
    
    if IterOutput > 0
        fprintf(' Starting search for RF peak power regularizer (initial Alpha = 10e%+.2f)\n',cur_alpha_power)
    end
    
    while Iter <= MaxWhileIter
        
        %update lambda, perform cg
        [RF_temp, flag1, Err]    = pcg( C + power(10,cur_alpha_power)^2 * eye(ncunk), d, tol, MaxCGIter, [], [], RF_0);
        RF_0       	= RF_temp;
        MaxVoltage1 = max(abs(RF_temp));
        FirstEmpty  = find( cellfun(@(c) isempty(c), RF_pulses_cell(:,1)),1,'first' );
        RF_pulses_cell{FirstEmpty,1} = power(10,cur_alpha_power);
        RF_pulses_cell{FirstEmpty,2} = RF_temp;
        RF_pulses_cell{FirstEmpty,3} = MaxVoltage1;
        
        if IterOutput > 0
            fprintf('#%3i/%i:   Vmax = %7.2f [V]   (Bound = %7.2f [V])   Alpha = 10e%+.2f\n', Iter, MaxWhileIter, MaxVoltage1, options.Vmax, cur_alpha_power)
        end
        
        %throw warnings
        switch flag1
            case 0
                %pcg converged to the desired tolerance tol within maxit iterations.
            case 1
                %pcg iterated maxit timesbut did not converge.
                warning('pcg iterated maxit times but did not converge.')
            case 2
                %Preconditioner M was ill-conditioned.
                warning('Preconditioner M was ill-conditioned.')
            case 3
                %pcg stagnated. (Two consecutive iterateswere the same.)
                warning('pcg stagnated. (Two consecutive iterateswere the same.)')
            case 4
                %One of the scalar quantities calculated during pcg becametoo small or too large to continue computing.
                warning('One of the scalar quantities calculated during pcg becametoo small or too large to continue computing.')
        end
        
        %run another pulse comp. to estimate derivative
        if Iter < MaxWhileIter && abs(MaxVoltage1 - options.Vmax) >= PeakRFtol
            [RF_temp, flag2, Err]    = pcg( C + power(10,cur_alpha_power + alpha_powerfindiff)^2 * eye(ncunk), d, tol, MaxCGIter, [], [], RF_0);
            RF_0       	= RF_temp;
            MaxVoltage2 = max(abs(RF_temp));
            FirstEmpty  = find( cellfun(@(c) isempty(c), RF_pulses_cell(:,1)),1,'first' );
            RF_pulses_cell{FirstEmpty,1} = power(10,cur_alpha_power + alpha_powerfindiff);
            RF_pulses_cell{FirstEmpty,2} = RF_temp;
            RF_pulses_cell{FirstEmpty,3} = MaxVoltage2;

            fprintf('FinDiff:   Vmax = %7.2f [V]   (Bound = %7.2f [V])   Alpha = 10e%+.2f\n', MaxVoltage2, options.Vmax, cur_alpha_power + alpha_powerfindiff)

            %throw warnings
            switch flag2
                case 0
                    %pcg converged to the desired tolerance tol within maxit iterations.
                case 1
                    %pcg iterated maxit timesbut did not converge.
                    warning('pcg iterated maxit times but did not converge.')
                case 2
                    %Preconditioner M was ill-conditioned.
                    warning('Preconditioner M was ill-conditioned.')
                case 3
                    %pcg stagnated. (Two consecutive iterateswere the same.)
                    warning('pcg stagnated. (Two consecutive iterateswere the same.)')
                case 4
                    %One of the scalar quantities calculated during pcg becametoo small or too large to continue computing.
                    warning('One of the scalar quantities calculated during pcg becametoo small or too large to continue computing.')
            end
        end

        if abs(MaxVoltage1 - options.Vmax) >= PeakRFtol && Iter < MaxWhileIter && abs( MaxVoltage1 - MaxVoltage2 ) > 0.1
            RF_final            = RF_temp;
            
            %update lambda, adjust for max step
            alpha_powerstep = ( MaxVoltage1 - options.Vmax )/( (MaxVoltage2 - MaxVoltage1)/alpha_powerfindiff );
            alpha_powerstep = sign(alpha_powerstep) * min([alpha_powermaxstep, abs(alpha_powerstep)]);
            
            cur_alpha_power     = cur_alpha_power - alpha_powerstep;
            Iter                = Iter + 1;
            Err_final           = Err;
        else         
            Err_final   = Err;
            if IterOutput == 1 
                fprintf('  Final:   Vmax = %7.2f [V]   (Bound = %7.2f [V])   Alpha = 10e%+.2f\n', MaxVoltage1, options.Vmax, cur_alpha_power)
                if abs(MaxVoltage1 - options.Vmax) < PeakRFtol 
                    fprintf('           (Peak RF tolerance satisfied!)\n')
                elseif    Iter >= MaxWhileIter
                    fprintf('           (Maximum number of iterations reached!)\n')
                else
                    fprintf('           (RF pulse seems to be unbound by peak RF!)\n')
                end
                fprintf('\n')
            end
            RF_final            = RF_temp;
            break;
        end

    end

    %final error handling
    if any(isnan(RF_final)) || any(isinf(RF_final))
       save('DesignPulseErrorData.mat','PulseOpt', 'C', 'd', 'RF_0', 'MaxWhileIter')
       error('RF waveform contains invalid entries!') 
    end
    
end
    
    