function [ funValue, J ] = EstimateJacobian( fun, x, options )

    %estimation of the Jacobian J using finite differences. Output J has
    %same size as x0 (length N), estimation requires N+1 function
    %evaluations
    
    %instead if using an output function within fmincon, the command output
    %is placed here to perform output at each function evaluation
    global CounterIter
    global CounterFunEval
    global DateString
    global run_alpha_update
    
    if ~isfield( options, 'optim_scalevariables' )
        options.optim_scalevariables = 0;
    end
    
    phi_bounds_red = options.phi_bounds( options.phi_isvariable > 0, : );
    
    if nargout < 2
        %call is used within line search
        if options.optim_scalevariables == 1
            %back out unscaled shape parameter
            objfundata  = fun( GetAbsoluteParameters( x, phi_bounds_red ), 1 );
        else
            %use variable vector as it is
            objfundata  = fun( x, 1 );
        end
        funValue    = objfundata.funValue;
        J           = [];
        CounterFunEval  = CounterFunEval + 1;
        
        %line search output here
        %-----------------------
        if isinf( objfundata.funValue )
            if ~isfield( objfundata, 'errmsg' )
                objfundata.errmsg = 'an unknown error occured!';
            end
            fprintf(2,'       Line Search aborted: %s\n', objfundata.errmsg )
        else
            if options.optim_scalevariables == 1
                xabs = GetAbsoluteParameters( x, phi_bounds_red );
                fprintf('       Line Search -    funValue: %.2f   |   NRMSE: %.2f   |   time: %6.3f ms   |   Phi = ( ', objfundata.funValue, objfundata.NRMSE, objfundata.time ), fprintf('%+5.2e, ',xabs(1:end-1)), fprintf('%+5.2e )\n',xabs(end))
            else
                fprintf('       Line Search -    funValue: %.2f   |   NRMSE: %.2f   |   time: %6.3f ms   |   Phi = ( ', objfundata.funValue, objfundata.NRMSE, objfundata.time ), fprintf('%+5.2e, ',x(1:end-1)), fprintf('%+5.2e )\n',x(end))
            end
        end
        return
    else
        
        %some first output
        fprintf('\n')
        fprintf('****************************************************************************************\n')
        fprintf('****************************************************************************************\n')
        fprintf('\n')
        
        %call is first call in new iteration (as derivative is computed as well)
        if options.optim_scalevariables == 1
            %back out unscaled shape parameter
            if exist( sprintf('SavedIterations/%s/iter_data.mat',DateString), 'file' ) && ~options.alpha_update
                prevdata = load(sprintf('SavedIterations/%s/iter_data.mat',DateString));
                %check if Phi is the same
                if all( prevdata.phi_red == GetAbsoluteParameters( x, phi_bounds_red ) )
                   objfundata = prevdata.objfundata;
                   fprintf(' Using objective function data from line search!\n')
                else
                   objfundata = fun( GetAbsoluteParameters( x, phi_bounds_red ), 1 );
                end
            elseif options.alpha_update && CounterIter > 0 && CounterIter <= options.optim_maxiter
                fprintf(' - - - - - - - - - -  U P D A T I N G   R F   R E G U L A R I Z E R   - - - - - - - - - - - - \n')
                fprintf('\n')
                %in this function call we want to adapt the regularization
                %parameter
                run_alpha_update    = true;
                objfundata          = fun( GetAbsoluteParameters( x, phi_bounds_red ), 0 );
                run_alpha_update    = false;
            else
                objfundata          = fun( GetAbsoluteParameters( x, phi_bounds_red ), 1 );
            end
        else
            %use variable vector as it is
            if exist( sprintf('SavedIterations/%s/iter_data.mat',DateString), 'file' ) && ~options.alpha_update
                prevdata = load(sprintf('SavedIterations/%s/iter_data.mat',DateString));
                %check if Phi is the same
                if all( prevdata.phi_red == x )
                   objfundata = prevdata.objfundata;
                   fprintf(' Using objective function data from line search!\n')
                else
                   objfundata = fun( x, 1 );
                end
         	elseif options.alpha_update && CounterIter > 0 && CounterIter <= options.optim_maxiter
                fprintf(' - - - - - - - - - -  U P D A T I N G   R F   R E G U L A R I Z E R   - - - - - - - - - - - - \n')
                fprintf('\n')
                %in this function call we want to adapt the regularization
                %parameter
                run_alpha_update    = true;
                objfundata = fun( x, 0 );
                run_alpha_update    = false;
            else
                objfundata = fun( x, 1 );
            end
        end
        funValue    = objfundata.funValue;
        CounterIter     = CounterIter + 1;
        CounterFunEval  = CounterFunEval + 1;
        
        %initial function evaluation output here
        %---------------------------------------
        
        if CounterIter <= options.optim_maxiter
        
            fprintf('      Iter: %3i         FunCount: %3i\n', CounterIter, CounterFunEval )
            fprintf('  Init. Evaluation -    funValue: %.2f   |   NRMSE: %.2f   |   time: %6.3f ms\n', objfundata.funValue, objfundata.NRMSE, objfundata.time )
            fprintf('\n')
        
            if options.optim_scalevariables == 1
                fprintf(' - - - - - - - - - - - -  R E L A T I V E    F I N I T E    D I F F   - - - - - - - - - - - - \n')
            else
                fprintf(' - - - - - - - - - - - - - - - -   F I N I T E    D I F F   - - - - - - - - - - - - - - - - - \n')
            end
        else
            fprintf('  Final Evaluation -    funValue: %.2f   |   NRMSE: %.2f   |   time: %6.3f ms\n', objfundata.funValue, objfundata.NRMSE, objfundata.time )
            fprintf('\n')
        end
        fprintf('\n')
    end
    
    %only compute Jacobian, if it is required
    if CounterIter <= options.optim_maxiter
        if options.optim_scalevariables == 1
            Phi_range = ones(size(x));
        else
            Phi_range = phi_bounds_red(:,2) - phi_bounds_red(:,1);
        end

        reldelta    = [1e-2, 1e-3, 1e-4]';
        mindiff     = 1e-4;
        J_mat       = nan( numel( x ), numel(reldelta) );
        Phi_idx     = find( options.phi_isvariable > 0 );

        for d = 1 : numel( reldelta )
            %fprintf('   Finite Differencing Rel. Delta:  %+.2e \n', reldelta(d) )
            for k = 1 : numel( x )

                %select step size
                if options.phi_isvariable( Phi_idx( k ) ) > 1
                    %use finer sampling in fin diff
                    delta   = Phi_range(k) * reldelta(d) * 1e-1;
                else
                    %use predefined sampling in fin diff
                    delta   = Phi_range(k) * reldelta(d);
                end

                %chose forward or backward differentiation
                if x(k) + delta > 1.0
                    delta   = -delta;
                end

                xk          = x;
                xk(k)       = xk(k) + delta;

                if options.optim_scalevariables == 1
                    %back out unscaled shape parameter
                    objfundata  = fun( GetAbsoluteParameters( xk, phi_bounds_red ), 0 );
                else
                    %use variable vector as it is
                    objfundata  = fun( xk, 0 );
                end

                dfunValue   = objfundata.funValue;
                if ~isinf(dfunValue)
                    J_mat(k,d)  = ( dfunValue - funValue )/delta;
                else
                    J_mat(k,d)  = 0.0;  
                end
                CounterFunEval  = CounterFunEval + 1;

                %finite differencing output here
                %-------------------------------
                if options.optim_scalevariables == 1
                    xabs = GetAbsoluteParameters( x, phi_bounds_red );
                    fprintf('   FinDiff Phi(%2i) -    funValue: %.2f   |   NRMSE: %.2f   |   time: %6.3f ms   |  df: %+5.2e   |  dx: %+4.1e   ( Phi: %+.2e )\n', Phi_idx(k), objfundata.funValue, objfundata.NRMSE, objfundata.time, J_mat(k,d), delta, xabs(k) )
                else
                    fprintf('   FinDiff Phi(%2i) -    funValue: %.2f   |   NRMSE: %.2f   |   time: %6.3f ms   |  df: %+5.2e   |  dx: %+4.1e   ( Phi: %+.2e )\n', Phi_idx(k), objfundata.funValue, objfundata.NRMSE, objfundata.time, J_mat(k,d), delta, x(k) )
                end

            end

            %check if Jacobian contains values significantly different from
            %zero; if yes exit loop, otherwise continue with smaller delta
            if any( abs( J_mat(:,d) ) >= mindiff )
               break 
            end
        end

        %extract Jacobian, assure correct formatting
        J = J_mat(:,d);
        if size( J,1 ) ~= size(x,1)
           J = J'; 
        end

        %some preparations for the upcoming line search function evaluations
        fprintf('\n')
        fprintf(' - - - - - - - - - - - - - - -   L I N E   S E A R C H   - - - - - - - - - - - - - - - \n')
        fprintf('\n')
    
    else
        J = zeros(size(x));
    end

end