function [ objfundata, T, RF, forwardmagn ] = GetFunValueForPhi( phi_red, options, b1mat, b0vec, saveiter_data )

    % This is the objective function.
    % In here, the RF pulses for given shape parameters are computed and
    % evaluated!

    clear functions

    global DateString
    global run_alpha_update
    
    if options.use_gpu
        reset(gpuDevice())
    end

    if nargin < 5
        saveiter_data = 0; 
    end

    % --------------------------------------------------------------
    % -----  T R Y   E V A L U A T I N G   O B J E C T I V E   -----
    % --------------------------------------------------------------
    try
        %retrieve some start values from the previous iteration
        if exist( sprintf('SavedIterations/%s/iter_data.mat',DateString), 'file' )
            Init    = load(sprintf('SavedIterations/%s/iter_data.mat',DateString));
            T_init  = Init.T;
            RF_init = Init.RF;
        else
            T_init  = [];
            RF_init	= [];
        end

        %back out full vector of shape parameters (assuming some of them 
        %are fixed)
        phi    = options.phi_init;
        phi( options.phi_isvariable > 0 ) = phi_red;
        
        % ADJUST TRAJECTORY
        %-------------------
        [ kx_vec, ky_vec, kz_vec, R_mat ] = feval( options.trajectory_fun, phi );
        
        %assure that no pathlogical control points are passed
        if any( isnan( [kx_vec(:); ky_vec(:); kz_vec(:)] ) ) || ...
           any( isinf( [kx_vec(:); ky_vec(:); kz_vec(:)] ) ) 
            warning('The set of shape parameters produced infeasible control points, a function value of +Inf will be returned!')
            fprintf('  phi = ( '), fprintf('%+5.2e, ',phi(1:end-1)), fprintf('%+5.2e )\n',phi(end))
            
            objfundata.RMSE     = NaN; 
            objfundata.NRMSE    = NaN;
            objfundata.time     = NaN;
            objfundata.SAR      = NaN;
            objfundata.funValue = +inf;
            objfundata.errmsg   = 'Shape parameters produced infeasible set of control points!';
            T                   = [];
            RF                  = [];
            forwardmagn         = [];
            
            return
        end

        
        %estimate pulse duration to avoid that excessively long
        %trajectotries (i.e., with a huge number of control points) are
        %computed. These trajectories are being rejected anyway
        time = EstimateTrajectoryDuration( kx_vec, ky_vec, kz_vec, R_mat, options.Gmax, options.Smax, [], options.GBF );
        if time > 1.5 * options.pulse_maxtime * 1e+3
            objfundata.RMSE     = NaN; 
            objfundata.NRMSE    = NaN;
            objfundata.time     = [];
            objfundata.SAR      = NaN;
            objfundata.funValue = +inf;
            objfundata.errmsg   = sprintf('Large pulse duration expected (1st iteration: t = %.2f ms)!',time); 
            T                   = [];
            RF                  = [];
            forwardmagn         = [];
            
            return
        end
        
        %get trajectory, perform hardware sampling
     	T   = PerformOptimization3D( kx_vec, ky_vec, kz_vec, R_mat, options.Gmax, options.Smax, [], options.GBF );
       
        %compute hardware gradients based on analytic trajectory
      	T   = CalculateRFTrajectory( T, options );
        
        if T.t(end) > options.pulse_maxtime * 1e+3
            objfundata.RMSE     = NaN; 
            objfundata.NRMSE    = NaN;
            objfundata.time     = T.t(end);
            objfundata.SAR      = NaN;
            objfundata.funValue = +inf;
            objfundata.errmsg   = sprintf('Pulse duration exceeds maximum tolerated duration (t = %.2f ms, t_max = %.2f ms)!',T.t(end),options.pulse_maxtime * 1e+3); 
            T                   = [];
            RF                  = [];
            forwardmagn         = [];
            
            return
        end

        % COMPUTE SYSTEM MATRIX
        %-----------------------
        Ind         = sub2ind([ options.nx, options.ny, options.nz ], options.xidx_all, options.yidx_all, options.zidx_all);
        coords    	= [ options.xpos_all(:), options.ypos_all(:), options.zpos_all(:) ];
        [ k_mat, zeroCols, T ] 	= PrepareSystemMatrixInput( T, options );

        %check if system matrix becomes to large, in this case, return
        %with function value +inf
        if size(coords,1) * options.nchannel * size(k_mat,1) * 16 > options.sysmat_maxsize
            %warning('The system matrix to be computed (%i x %i) exceeds the specified maximum memory size (%.1e > %.1e)! A function value of +Inf will be returned!',size(coords,1), options.NumChannel * size(k_mat,1), size(coords,1) * options.NumChannel * size(k_mat,1) * 16, options.SysMaxSize)
            objfundata.RMSE     = NaN; 
            objfundata.NRMSE    = NaN;
            objfundata.time     = T.t(end);
            objfundata.SAR      = NaN;
            objfundata.funValue = +inf;
            objfundata.errmsg   = sprintf('System matrix to be computed (%i x %i) exceeds the specified maximum memory size (actual size: %.1f MB, tolerated: %.1f MB)!',size(coords,1), options.nchannel * size(k_mat,1), size(coords,1) * options.nchannel * size(k_mat,1) * 2 / 1e+6, options.sysmat_maxsize / 1e+6);
            T                   = [];
            RF                  = [];
            forwardmagn         = [];
            return
        end
            
        %when using MLS we need to compute another system matrix 
        if options.enforceB0 ~= 0
            A = [   ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec - options.enforceB0 ); ...    
                    ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec ); ...   
                    ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec + options.enforceB0 ) ];
            targetmagn_mat = repmat( options.targetmagn_mat( Ind ), 3, 1 );
        else
            A =     ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec );
            targetmagn_mat = options.targetmagn_mat( Ind );
        end

        A( :, zeroCols )    = 0.0;
       
        % ASSOCIATE RF-PULSE or RECOMPUTE
        %---------------------------------

        %use old RF, with new RF. This method is a little bit unstable; in case
        %it fails, simply set the initial guess RF to empty
        try
            if ~isempty( RF_init ) && ~isempty( T_init )
                RF_full_0               = GetFullRF( RF_init, T_init );
                RF_full                 = InterpolateTrajectoryRF( T_init, RF_full_0, T, options.nchannel );
                RF_init                 = GetSparseRF( RF_full, T );
                RF_init(isnan(RF_init))	= 0.0;
            else
                RF_init                 = [];
            end
        catch exception
            warning('WarnTests:convertTest','Interpolation of the RF waveform failed, no initial guess will be used within the RF solver!')
            fprintf('Reason: %s\n', exception.message)
            RF_init	= [];
        end
        
        switch options.optim_rfsolver
            case 1
                % Use Conjugate Gradient Solver
                %...............................

                % set up system matrix for cg
                if options.use_gpu == 1
                    %use GPU to set up the matrices required in pcg
                    try
                        A_gpu           	= gpuArray( A );
                        targetmagn_mat_gpu  = gpuArray( targetmagn_mat );
                        C                   = gather( A_gpu' * A_gpu );
                        d                   = gather( A_gpu' * targetmagn_mat_gpu );
                        clear A_gpu targetmagn_mat_gpu
                    catch exception
                        % warning('WarnTests:convertTest','Cannot use GPU, not enough memory!')
                        % fprintf('Reason: %s\n', exception.message)
                        C  	= A' * A;
                        d  	= A' * targetmagn_mat;
                    end
                else
                    C 	= A' * A;
                    d 	= A' * targetmagn_mat;
                end

                % solve RF pulse, perform search for alpha
                if run_alpha_update
                    %re/run search for regularizer
                    [ RF, solres ] = DesignPulse_ConjugateGradient( options, C, d, RF_init, 10 );
                else
                    %use predefined regularizer
                    [ RF, solres ] = DesignPulse_ConjugateGradient( options, C, d, RF_init );
                end
        
            case 2
                % Use Conjugate Gradient Solver
                %...............................

                [ RF, solres ] = DesignPulse_LeastSquares( options, A, targetmagn_mat, RF_init );     
                
        end
        
        % COMPUTE FORWARD MAGNETIZATION
        %-------------------------------
        if options.enforceB0 ~= 0
            forwardmagn     = CalculateMagnetization( options, A( 1 : options.npixel, : ), RF );
        else
            forwardmagn     = CalculateMagnetization( options, A, RF );
        end
        
        [ objfundata.RMSE, objfundata.NRMSE ] = ComputeForwardError( A, RF, targetmagn_mat );
        
        %set other entries of objfundata
        objfundata.solres   = solres;
        objfundata.time     = T.t(end);
        objfundata.SAR      = NaN;
        objfundata.errmsg   = [];

        %check if all fields are contained in options.ObjData
        objfundata.funValue = RegularizedObjectiveFunction( objfundata, options );

        if saveiter_data == 1
            %store some data for plotting
            save(sprintf('SavedIterations/%s/plot_data.mat',DateString),'T','RF','forwardmagn','options','objfundata');
            
            %save start value for the next iteration
            save(sprintf('SavedIterations/%s/iter_data.mat',DateString),'T','RF','forwardmagn','phi_red','objfundata');
        end
        
    % ------------------------------------------------------------------
    % ----------------  E R R O R     H A N D L I N G   ----------------
    % ------------------------------------------------------------------
    catch exception
        switch options.optim_errhandling
            case 1
                %stop iteration, throw error
                rethrow( exception )
                
            case 2
                %an error occured, return infinite functional value, save input
                %files
                objfundata.RMSE     = NaN;
                objfundata.NRMSE    = NaN;
                objfundata.time     = NaN;
                objfundata.SAR      = NaN;
                objfundata.funValue = NaN;
                objfundata.errmsg   = sprintf('Error occured (%s)!',exception.message);
                T                   = [];
                RF                  = []; 
                forwardmagn         = [];

                %prepare saving file for error search
                Counter         = 0;
                CounterString   = '';
                while exist( sprintf('SavedIterations/%s/Error%s.mat',DateString,CounterString), 'file' )
                    Counter         = Counter + 1;
                    CounterString   = sprintf('_%i',Counter);
                end

                warning('WarnTests:convertTest','An error occured in evaluating the functional. The iteration continues, but an error file will be saved at ''%s''', sprintf('SavedIterations/%s/Error%s.mat',DateString,CounterString))
                fprintf('Reason: %s\n', exception.message)
                save( sprintf('SavedIterations/%s/Error%s.mat',DateString,CounterString), 'exception', 'phi', 'T_init', 'RF_init', 'options' )
        end
    end
    