function T = ComputeAnalyticGradients( T, Smax_adj )

    %this method recalculates the analytic gradients from the Gx_opt and
    %Gy_opt vectors stored in the trajectory struct
    
    if nargin < 2
        Smax_adj = ones(numel(T.Ana.Gx_opt)-1,3) .* T.Smax;
    else
        %check for correct format
        if size(Smax_adj,1) ~= numel(T.Ana.Gx_opt) - 1 || ...
           size(Smax_adj,2) ~= 3
            error('The Smax_adj matrix has wrong dimension!')
        end
    end
    
    %error handling
    if ~isfield(T.Ana,'Gx_opt') || ~isfield(T.Ana,'Gy_opt')
       error('No optimal node gradients stored in T!') 
    end
    
    if numel(T.Ana.Gx_opt) ~= numel(T.Ana.Gy_opt)
       error('Optimal node gradient vectors need to have same length!') 
    end
    
    %if no kz infos are available set to zeros
    if ~isfield(T,'kz')
    	T.kz            = zeros(size(T.kx));
        T.Ana.Gz_opt    = zeros(size(T.Ana.Gx_opt));
    end
    
    %read needed variables
    dkx         = T.kx(2:end) - T.kx(1:end-1);
    dky         = T.ky(2:end) - T.ky(1:end-1);
    dkz         = T.kz(2:end) - T.kz(1:end-1);
    Gx_opt      = T.Ana.Gx_opt;
    Gy_opt      = T.Ana.Gy_opt;
    Gz_opt      = T.Ana.Gz_opt;
    Gmax        = T.Gmax;
    
    %delete existing entrys
    T.Ana.Gx    = [];   T.Ana.Gy    = [];   T.Ana.Gz    = [];
    T.Ana.Sx    = [];   T.Ana.Sy    = [];   T.Ana.Sz    = []; 
    T.Ana.tx    = [];   T.Ana.ty    = [];   T.Ana.tz    = [];
    T.t         = [];
    
    T.Sam.Gx    = [];   T.Sam.Gy    = [];   T.Sam.Gz    = [];
    T.Sam.Sx    = [];   T.Sam.Sy    = [];   T.Sam.Sz    = []; 
    T.Sam.kx    = [];   T.Sam.ky    = [];   T.Sam.kz    = []; 
    T.Sam.t     = [];
 
    tlim_sum    = 0.0;
    for l = 1 : numel(Gx_opt)-1
        [tlim,Gx,Gy,Gz,Sx,Sy,Sz,tx,ty,tz] ...
            = FullTimeOptGrad3D(Gx_opt(l), Gx_opt(l+1), ...
                                Gy_opt(l), Gy_opt(l+1), ...
                                Gz_opt(l), Gz_opt(l+1), ...
                                dkx(l), ...
                                dky(l), ...
                                dkz(l), ...
                                Gmax, Smax_adj(l,:) );
        tlim_sum = tlim_sum + tlim;

        if numel(unique([numel(Gx), numel(Sx), numel(tx)])) > 1 || ...
           numel(unique([numel(Gy), numel(Sy), numel(ty)])) > 1 || ...
           numel(unique([numel(Gz), numel(Sz), numel(tz)])) > 1
            error('Length missmatch of analytic gradient waveform found!')
        end
        
        %remove imaginary parts
        Gx = real(Gx);
        Gy = real(Gy);
        Gz = real(Gz);
        tx = real(tx);
        ty = real(ty);
        tz = real(tz);

        %append solved gradient basis functions to T struct
        T.t(l+1) = tlim_sum;
        T = AppendAnalyticGradient3D( T,    Gx, ...   	%Gx
                                            Gy, ...    	%Gy
                                            Gz, ...    	%Gz
                                            Sx, ...   	%Sx
                                            Sy, ...    	%Sy
                                            Sz, ...    	%Sz
                                            tx, ...    	%tx
                                            ty, ...    	%ty
                                            tz, ...    	%tz
                                            'relative' );  

    end
    
end
    
function T = AppendAnalyticGradient3D( T, Gx, Gy, Gz, Sx, Sy, Sz, tx, ty, tz, Type )

    %this method is used to append a gradient part to the current analytic
    %gradients in T
    %
    % Type = 'relative' --> relative time values
    % Type = 'absolute' --> absolute time values
    
    %error handling
    %check if all inputs have vector format
    if size(Gx,1) ~= numel(Gx) && size(Gx,2) ~= numel(Gx)
       error('Gx input is not a vector') 
    end
    
    if size(Gy,1) ~= numel(Gy) && size(Gy,2) ~= numel(Gy)
       error('Gy input is not a vector') 
    end
    
    if size(Gz,1) ~= numel(Gz) && size(Gz,2) ~= numel(Gz)
       error('Gz input is not a vector') 
    end
    
    if size(Sx,1) ~= numel(Sx) && size(Sx,2) ~= numel(Sx)
       error('Gx input is not a vector') 
    end
    
    if size(Sy,1) ~= numel(Sy) && size(Sy,2) ~= numel(Sy)
       error('Gy input is not a vector') 
    end
    
    if size(Sz,1) ~= numel(Sz) && size(Sz,2) ~= numel(Sz)
       error('Gy input is not a vector') 
    end
    
    if size(tx,1) ~= numel(tx) && size(tx,2) ~= numel(tx)
       error('tx input is not a vector') 
    end
    
    if size(ty,1) ~= numel(ty) && size(ty,2) ~= numel(ty)
       error('ty input is not a vector') 
    end
    
    if size(tz,1) ~= numel(tz) && size(tz,2) ~= numel(tz)
       error('ty input is not a vector') 
    end
    
    %check if Gx/tx and Gy/tx are of same length
    if numel(Gx) ~= numel(tx)
       error('Gx and tx are of different length') 
    end
    
    if numel(Gy) ~= numel(ty)
       error('Gx and tx are of different length') 
    end
    
    if numel(Gz) ~= numel(tz)
       error('Gz and tz are of different length') 
    end
    
    %change formats
    if size(Gx,2) ~= numel(Gx)
       Gx = Gx';
    end
    
    if size(Gy,2) ~= numel(Gy)
       Gy = Gy';
    end
    
    if size(Gz,2) ~= numel(Gz)
       Gz = Gz';
    end
    
  	if size(tx,2) ~= numel(tx)
       tx = tx';
    end
    
    if size(ty,2) ~= numel(ty)
       ty = ty';
    end
    
    if size(tz,2) ~= numel(tz)
       tz = tz';
    end
    
    %check if last time points are equal
    if tx(end) ~= ty(end) || tx(end) ~= tz(end)
       warning('tx, ty, and/or tz have different end times') 
    end

    %append gradients
    if      strcmp(Type,'relative') && ~isempty(T.Ana.tx) && ~isempty(T.Ana.ty) && ~isempty(T.Ana.tz)
        tx = tx + T.Ana.tx(end);
        ty = ty + T.Ana.ty(end);
        tz = tz + T.Ana.tz(end);
    elseif  ~(strcmp(Type,'absolute') || strcmp(Type,'relative'))
        error('Type identifier undefined') 
    end
    
    T.Ana.Gx = [T.Ana.Gx, Gx];
    T.Ana.Sx = [T.Ana.Sx, Sx];
    T.Ana.tx = [T.Ana.tx, tx];
    
    T.Ana.Gy = [T.Ana.Gy, Gy];
    T.Ana.Sy = [T.Ana.Sy, Sy];
    T.Ana.ty = [T.Ana.ty, ty];
    
    T.Ana.Gz = [T.Ana.Gz, Gz];
    T.Ana.Sz = [T.Ana.Sz, Sz];
    T.Ana.tz = [T.Ana.tz, tz];

end