function tlim = FastTimeOptGrad_2D_deriv0( GxIn, GxOut, GyIn, GyOut, dkx, dky, Gmax, Smax, OptsVector )

    %# codegen
    %this method is used to solver a local trajectory segment in 2D
    %dimension, based on the gradient vectors in the control points. tlim
    %contains the smallest possible duration of the local segment
    %satisfying the gradient vectors as well as the Gradient constraints
    %Gmax and Smax

    %check for correct input values
    if abs(GxIn) > Gmax
        GxIn = Gmax * sign(GxIn);
    end
    if abs(GyIn) > Gmax
        GyIn = Gmax * sign(GyIn);
    end
    if abs(GxOut) > Gmax
        GxOut = Gmax * sign(GxOut);
    end
    if abs(GyOut) > Gmax
        GyOut = Gmax * sign(GyOut);
    end

    %valid time ranges in X
    [ lowBoundsX, uppBoundsX, ~ ] = GetValidTimeRanges( GxIn, GxOut, dkx, Gmax, Smax );
    %valid time ranges in X
    [ lowBoundsY, uppBoundsY, ~ ] = GetValidTimeRanges( GyIn, GyOut, dky, Gmax, Smax );
    
    if OptsVector( 1 )
        %remove parts of range making the full range incontinous
       [ lowBoundsX, uppBoundsX ] = MakeRangeContinous( lowBoundsX, uppBoundsX ); 
       [ lowBoundsY, uppBoundsY ] = MakeRangeContinous( lowBoundsY, uppBoundsY ); 
    end
    
    %get tlim
    tlim = GetMinimumTime2D( lowBoundsX, uppBoundsX, lowBoundsY, uppBoundsY, OptsVector );
   
end

function [ C_lowBounds, C_uppBounds, C_solIndex ] = GetValidTimeRanges( GnIn, GnOut, dkn, Gmax, Smax )
  
    %error handling
    if abs(GnIn) > Gmax
        error('GnIn exceeds Gmax constraint')
    end
    
    if abs(GnOut) > Gmax
        error('GnOut exceeds Gmax constraint')
    end

    %warning('Gamma changed!')
    %Gamma       = 2.675222005e+2;
    Gamma   = 42.5775;
    %Gamma = 1;
    dknGamma    = dkn/Gamma;

    %final ranges
    %up to 5 overall ranges might be given for the solution, 2 ranges for the
    %fast convex or concave shape, 1 for the linear shape and 2 for the slow
    %convex or concave shape

    R1_lowBound     = NaN;
    R1_uppBound     = NaN;
    R1_solIndex     = NaN;
    
    R2_lowBound     = NaN;
    R2_uppBound     = NaN;
    R2_solIndex     = NaN;
    
    R3_lowBound     = NaN;
    R3_uppBound     = NaN;
    R3_solIndex     = NaN;
    
    R4_lowBound     = NaN;
    R4_uppBound     = NaN;
    R4_solIndex     = NaN;
    
    R5_lowBound     = NaN;
    R5_uppBound     = NaN;
    R5_solIndex     = NaN;

    t_const     = abs(GnIn-GnOut)/Smax;
    I_const 	= (sign(GnIn-GnOut)*(GnIn^2-GnOut^2)*(1.0/2.0))/Smax;
    I_rest      = -I_const+dknGamma;
    
    %special case, GnIn =0, GnOut = 0, dkn = 0
    if GnIn == 0 && GnOut == 0 && dkn == 0
        C_lowBounds     = 0;
        C_uppBounds     = Inf;
        C_solIndex      = 1;
        return
    end
    
    %get adjusted sign parameter
  	AdjustedSign = sign(GnIn - GnOut);
    if AdjustedSign == 0
        %in case AdjustedSign == 0, i.e. GnIn == GnOut, we have to decide, 
        %as how to treat the problem 
        switch sign(I_rest)
            case +1
                AdjustedSign = -1; 
            case -1
                AdjustedSign = +1;
            case 0
                AdjustedSign = +1; 
        end
        
    end

    %constant integral given be the connection of GnIn and GnOut
    switch AdjustedSign
        case -1
            %GnIn <= GnOut --> Increasing Shape

            %in the GnIn <= GnOut, the problem cannot be solved by the mono.
            %decreasing gradient type
            F2_lowBound     = NaN;
            F2_uppBound     = NaN;

            %depending on the signs of GnIn and GnOut, the range of the mono.
            %increasing gradient type is defined
            GnIn_Bound      = I_rest / GnIn;
            GnOut_Bound     = I_rest / GnOut;
            
            if isnan(GnIn_Bound)
                GnIn_Bound = 0.0;
            end
            if isnan(GnOut_Bound)
                GnOut_Bound = 0.0;
            end

            if      sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == +1
                F1_lowBound     = t_const+min([GnIn_Bound,GnOut_Bound]);
                F1_uppBound     = t_const+max([GnIn_Bound,GnOut_Bound]);

            elseif  sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == -1
                F1_lowBound     = GnIn_Bound+t_const;
                F1_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == +1
                F1_lowBound     = GnOut_Bound+t_const;
                F1_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == -1
                F1_lowBound     = NaN;
                F1_uppBound     = NaN;

            elseif  I_rest == 0
                if sign(GnIn) < sign(GnOut) || sign(GnIn) > sign(GnOut)
                    F1_lowBound     = t_const;
                    F1_uppBound     = Inf;
                else
                    F1_lowBound     = t_const;
                    F1_uppBound     = t_const;
                end
            else
                error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
            end

        case +1
            %GnIn >= GnOut --> Decreasing Shape
            I_rest   = -I_const+dknGamma;

            %in the GnIn <= GnOut, the problem cannot be solved by the mono.
            %decreasing gradient type
            F1_lowBound     = NaN;
            F1_uppBound     = NaN;

            %depending on the signs of GnIn and GnOut, the range of the mono.
            %increasing gradient type is defined
            GnIn_Bound      = I_rest / GnIn;
            GnOut_Bound     = I_rest / GnOut;

            if      sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == +1
                F2_lowBound     = t_const+min([GnIn_Bound,GnOut_Bound]);
                F2_uppBound     = t_const+max([GnIn_Bound,GnOut_Bound]);

            elseif  sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == -1
                F2_lowBound     = GnIn_Bound+t_const;
                F2_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == +1
                F2_lowBound     = GnOut_Bound+t_const;
                F2_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == -1
                F2_lowBound     = NaN;
                F2_uppBound     = NaN;

            elseif  I_rest == 0
                if sign(GnIn) < sign(GnOut) || sign(GnIn) > sign(GnOut)
                    F2_lowBound     = t_const;
                    F2_uppBound     = Inf;
                else
                    F2_lowBound     = t_const;
                    F2_uppBound     = t_const;
                end
            else
            	error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)

            end

        otherwise
            error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
    end

    %fprintf('F1-Bounds (increasing):   [%.4f,...,%.4f]\n',F1_lowBound,F1_uppBound)
    %fprintf('F2-Bounds (decreasing):   [%.4f,...,%.4f]\n',F2_lowBound,F2_uppBound)

    %Get concave and convex parameters by subfunction
    switch AdjustedSign
        case -1
            %GnIn <= GnOut --> Increasing Shape
            switch sign(I_rest)
                case -1

                    % FAST:     F3 Convex shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F4 Concave shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F3_lowBoundA;
                    R1_uppBound     = F3_uppBoundA;
                    R1_solIndex     = 3;
                    
                    R2_lowBound     = F3_lowBoundB;
                    R2_uppBound     = F3_uppBoundB;
                    R2_solIndex     = 3;

                    %linear
                    R3_lowBound     = F1_lowBound;
                    R3_uppBound     = F1_uppBound;
                    R3_solIndex     = 1;

                    %slow
                    R4_lowBound     = F4_lowBoundA;
                    R4_uppBound     = F4_uppBoundA;
                    R4_solIndex     = 4;
                    
                    R5_lowBound     = F4_lowBoundB;
                    R5_uppBound     = F4_uppBoundB;
                    R5_solIndex     = 4;

                case +1

                    % FAST:     F4 Concave shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F3 Convex shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F4_lowBoundA;
                    R1_uppBound     = F4_uppBoundA;
                    R1_solIndex     = 4;
                    
                    R2_lowBound     = F4_lowBoundB;
                    R2_uppBound     = F4_uppBoundB;
                    R2_solIndex     = 4;

                    %linear
                    R3_lowBound     = F1_lowBound;
                    R3_uppBound     = F1_uppBound;
                    R3_solIndex     = 1;

                    %slow
                    R4_lowBound     = F3_lowBoundA;
                    R4_uppBound     = F3_uppBoundA;
                    R4_solIndex     = 3;
                    
                    R5_lowBound     = F3_lowBoundB;
                    R5_uppBound     = F3_uppBoundB;
                    R5_solIndex     = 3;
                    
           	case 0
                %no residual integration; I_rest = 0
                    if (sign(GnIn) == -1 && sign(GnOut) == +1) || ...
                       (sign(GnIn) == +1 && sign(GnOut) == -1)  
                            %set full increasing
                            R1_lowBound     = F1_lowBound;
                            R1_uppBound     = F1_uppBound;
                            R1_solIndex     = 1;
                    else
                    
                        if abs(GnIn) <= abs(GnOut)
                            GnOffset    = GnIn;
                        else
                            GnOffset    = GnOut;
                        end

                        %Depending on the sign of GnOffset, a convex or concave
                        %shape will be used

                        [ F34_lowBoundA, F34_uppBoundA, ~, ~ ] ...
                            = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax );

                        %no faster solution than increasing possible
                        R1_lowBound     = NaN;
                        R1_uppBound     = NaN;
                        R1_solIndex     = NaN;
                        
                        R2_lowBound     = NaN;
                        R2_uppBound     = NaN;
                        R2_solIndex     = NaN;

                        R3_lowBound     = F1_lowBound;
                        R3_uppBound     = F1_uppBound;
                        R3_solIndex     = 1;

                        R4_lowBound     = F34_lowBoundA;
                        R4_uppBound     = F34_uppBoundA;

                        R5_lowBound     = NaN;
                        R5_uppBound     = NaN;
                        R5_solIndex     = NaN;

                        %Set basis function identifier
                        switch sign(GnOffset)
                            case -1
                                %Use Concave, if GnOffset < 0
                                R4_solIndex     = 4;
                            case +1
                                %Use Convex, if GnOffset > 0
                                R4_solIndex     = 3;
                            case 0
                                %Unsolvable, if GnOffset == 0
                                %R3_solIndex     = 1;
                                R4_solIndex     = NaN;
                        end
                    
                    end
            end

        case +1
            %GnIn >= GnOut --> Decreasing Shape
            switch sign(I_rest)
                case -1
                    % FAST:     F4 Convex shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F3 Concave shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F4_lowBoundA;
                    R1_uppBound     = F4_uppBoundA;
                    R1_solIndex     = 3;
                    
                    R2_lowBound     = F4_lowBoundB;
                    R2_uppBound     = F4_uppBoundB;
                    R2_solIndex     = 3;

                    %linear
                    R3_lowBound     = F2_lowBound;
                    R3_uppBound     = F2_uppBound;
                    R3_solIndex     = 2;

                    %slow
                    R4_lowBound     = F3_lowBoundA;
                    R4_uppBound     = F3_uppBoundA;
                    R4_solIndex     = 4;
                    
                    R5_lowBound     = F3_lowBoundB;
                    R5_uppBound     = F3_uppBoundB;
                    R5_solIndex     = 4;

                case +1

                    % FAST:     F3 Convex shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F4 Concave shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F3_lowBoundA;
                    R1_uppBound     = F3_uppBoundA;
                    R1_solIndex     = 4;
                    
                    R2_lowBound     = F3_lowBoundB;
                    R2_uppBound     = F3_uppBoundB;
                    R2_solIndex     = 4;

                    %linear
                    R3_lowBound     = F2_lowBound;
                    R3_uppBound     = F2_uppBound;
                    R3_solIndex     = 2;

                    %slow
                    R4_lowBound     = F4_lowBoundA;
                    R4_uppBound     = F4_uppBoundA;
                    R4_solIndex     = 3;
                    
                    R5_lowBound     = F4_lowBoundB;
                    R5_uppBound     = F4_uppBoundB;
                    R5_solIndex     = 3;
                    
         	case 0
                %no residual integration; I_rest = 0
                    if (sign(GnIn) == -1 && sign(GnOut) == +1) || ...
                       (sign(GnIn) == +1 && sign(GnOut) == -1)  
                            %set full decreasing
                            R1_lowBound     = F2_lowBound;
                            R1_uppBound     = F2_uppBound;
                            R1_solIndex     = 2;
                    else
                    
                        if abs(GnIn) <= abs(GnOut)
                            GnOffset    = GnIn;
                        else
                            GnOffset    = GnOut;
                        end

                        %Depending on the sign of GnOffset, a convex or concave
                        %shape will be used

                        [ F34_lowBoundA, F34_uppBoundA, ~, ~ ] ...
                            = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax );

                        %no faster solution than increasing possible
                        R1_lowBound     = NaN;
                        R1_uppBound     = NaN;
                        R1_solIndex     = NaN;
                        
                        R2_lowBound     = NaN;
                        R2_uppBound     = NaN;
                        R2_solIndex     = NaN;

                        R3_lowBound     = F2_lowBound;
                        R3_uppBound     = F2_uppBound;
                        R3_solIndex     = 2;

                        R4_lowBound     = F34_lowBoundA;
                        R4_uppBound     = F34_uppBoundA;

                        R5_lowBound     = NaN;
                        R5_uppBound     = NaN;
                        R5_solIndex     = NaN;

                        %Set basis function identifier
                        switch sign(GnOffset)
                            case -1
                                %Use Concave, if GnOffset < 0
                                R4_solIndex     = 4;
                            case +1
                                %Use Convex, if GnOffset > 0
                                R4_solIndex     = 3;
                            case 0
                                %Unsolvable, if GnOffset == 0
                                %R3_solIndex     = 1;
                                R4_solIndex     = NaN;
                        end
                    
                    end     
            end
    end

    %Get concatenated ranges
    C_lowBounds     = nan(5,1);
    C_uppBounds     = nan(5,1);
    C_solIndex      = nan(5,1);

    while(1)
%         %special case
%         if R3_lowBound == R3_uppBound && 
%             C_lowBounds(1) = R1_lowBound;
%             C_uppBounds(1) = R1_uppBound;
%             C_solIndex(1)  = R1_solIndex;
%             break; 
%         end
        
        %special case
%         if R1_lowBound > 0.0 && isinf(R1_uppBound)
%             C_lowBounds(1) = R1_lowBound;
%             C_uppBounds(1) = R1_uppBound;
%             C_solIndex(1)  = R1_solIndex;
%             break; 
%         end
        
    %special case if I_rest == 0, GnIn == GnOut == 0
%     if I_rest == 0 && GnIn == 0 && GnOut == 0
%         warning('_rest == 0 && GnIn == 0 && GnOut == 0')
%         C_lowBounds(1) = 0;
%         C_uppBounds(1) = +Inf;
%         C_solIndex(1)  = 1;
%         break;
%     end

    %special case if I_rest == 0, GnIn == GnOut
%     if I_rest == 0 && GnIn == GnOut
%         warning('I_rest == 0 && GnIn == GnOut')
%         %only a single of time point that solves
%         C_lowBounds(1) = R3_lowBound;
%         C_uppBounds(1) = R3_uppBound;
%         C_solIndex(1)  = R3_solIndex;
%         
%         if R1_lowBound ~= R4_lowBound || R1_uppBound ~= R4_uppBound || ...
%            (R2_lowBound ~= R5_lowBound && ~isnan(R2_lowBound) && ~isnan(R5_lowBound)) || ...
%            (R2_uppBound ~= R5_uppBound && ~isnan(R2_uppBound) && ~isnan(R5_uppBound))
%                 %different intervals, choose smaller one
%                 error('Undefined case')
%         end
%         
%         %set conventional range
%         C_lowBounds(2) = R1_lowBound;
%         C_uppBounds(2) = R1_uppBound;
% 
%        	C_lowBounds(3) = R2_lowBound;
%        	C_uppBounds(3) = R2_uppBound;
%         
%         %set function type manually
%         if      GnIn > 0 
%             %function has to be convex
%             C_solIndex(2)   = 3;
%             if ~isnan(R2_lowBound)
%                 C_solIndex(3)   = 3;
%             end
%             
%         elseif  GnIn < 0 
%             %function has to be concave
%             C_solIndex(2)   = 4;
%             if ~isnan(R2_lowBound)
%                 C_solIndex(3)   = 4;
%             end
%             
%         end
%         break;
%     end
    
    %special case: just zero residual integral
%     if I_rest   == 0
%         %single of time point that solves
%         C_lowBounds(1) = R3_lowBound;
%         C_uppBounds(1) = R3_uppBound;
%         C_solIndex(1)  = R3_solIndex;
%         
%         %choose smaller time interval
%         if R1_lowBound <= R4_lowBound
%             
%             C_lowBounds(2) = R1_lowBound;
%             C_uppBounds(2) = R1_uppBound;
%             C_solIndex(2)  = R1_solIndex;
%             if ~isnan(R2_lowBound)
%                 C_lowBounds(3) = R2_lowBound;
%                 C_uppBounds(3) = R2_uppBound;
%                 C_solIndex(3)  = R2_solIndex;
%             end
%             
%         else
%             
%             C_lowBounds(2) = R4_lowBound;
%             C_uppBounds(2) = R4_uppBound;
%             C_solIndex(2)  = R4_solIndex;
%             if ~isnan(R5_lowBound)
%                 C_lowBounds(3) = R5_lowBound;
%                 C_uppBounds(3) = R5_uppBound;
%                 C_solIndex(3)  = R5_solIndex;
%             end
%              
%         end
%         break;
%     end

%         %special case, I_rest == 0
%         if R3_lowBound == R3_uppBound
%             C_lowBounds(1) = R3_lowBound;
%             C_uppBounds(1) = R3_uppBound;
%             C_solIndex(1)  = R3_solIndex;
% 
%             %use smallest convex or concave solution
%             if      R1_lowBound <= R4_lowBound
%                 C_lowBounds(2)  = R1_lowBound;
%                 C_uppBounds(2)  = R1_uppBound;
%                 C_solIndex(2)   = R1_solIndex;
%                 
%                 %set divided ranges
%                 if ~isnan(R2_lowBound)
%                     C_lowBounds(3)  = R2_lowBound;
%                     C_uppBounds(3)  = R2_uppBound;
%                     C_solIndex(3)   = R2_solIndex;
%                 end
%                     
%                 break;
%             elseif  R1_lowBound > R4_lowBound
%                 C_lowBounds(2)  = R4_lowBound;
%                 C_uppBounds(2)  = R4_uppBound;
%                 C_solIndex(2)   = R4_solIndex;
%                 
%                 if ~isnan(R5_lowBound)
%                     C_lowBounds(3)  = R5_lowBound;
%                     C_uppBounds(3)  = R5_uppBound;
%                     C_solIndex(3)   = R5_solIndex;
%                 end
%                 
%                 break;
%             end
%             error('Error in I_rest == 0 case')
%         end

        %Set low limits of part 1
        C_lowBounds(1) = R1_lowBound;
        C_uppBounds(1) = R1_uppBound;
        C_solIndex(1)  = R1_solIndex;

        %return, if upper bound is inf
        if isinf(R1_uppBound) && isnan(R3_lowBound) || ...
           isinf(R1_uppBound) && isinf(R3_lowBound)     
            break;
        end

        if      R1_uppBound >= R3_lowBound
            C_uppBounds(1) = R3_lowBound;
        elseif  R2_lowBound >= R3_lowBound
            C_lowBounds(2) = R2_lowBound;
            C_uppBounds(2) = R2_uppBound;
            C_solIndex(2)  = R2_solIndex;
            %again, adjust upper
            if R2_uppBound >= R3_lowBound
                C_uppBounds(2) = R3_lowBound;
            end
        end
        
        if isinf(R2_uppBound) && isnan(R3_lowBound) && isnan(R3_uppBound)
            C_lowBounds(2) = R2_lowBound;
            C_uppBounds(2) = R2_uppBound;
            C_solIndex(2)  = R2_solIndex;
            break;
        end

        if isnan(R3_lowBound) && isnan(R3_uppBound)
           break; 
        end
        
        %set linear phase bounds
        C_lowBounds(3) = R3_lowBound;
        C_uppBounds(3) = R3_uppBound;
        C_solIndex(3)  = R3_solIndex;

        %return, if upper bound is inf
        if isinf(R3_uppBound)
            break;
        end 

        %Set low limits of part 3
        C_lowBounds(4) = R4_lowBound;
        C_uppBounds(4) = R4_uppBound;
        C_solIndex(4)  = R4_solIndex;

        %adjust to linear bounds
        if R4_lowBound < R3_uppBound
            C_lowBounds(4) = R3_uppBound;
        end

        %return, if upper bound is inf
        if isinf(R4_uppBound)
            break;
        end 

        %Set low limits of part 3
        C_lowBounds(5) = R5_lowBound;
        C_uppBounds(5) = R5_uppBound;
        C_solIndex(5)  = R5_solIndex;

        %return, if upper bound is inf
        if isinf(R5_uppBound)
            break;
        end 

        error('Limited time soltution occurred!')

    end

    %unequal nans in time ranges
    if min(isnan(C_lowBounds) == isnan(C_uppBounds) == 0)
        error('NaN occurred in time ranges!')
    end

    C_lowBounds(isnan(C_lowBounds)) = [];
    C_uppBounds(isnan(C_uppBounds)) = [];
    C_solIndex(isnan(C_solIndex))   = [];
    
end

%Subfunction to deal with the convex and concave parts
function [ lowBoundA, uppBoundA, lowBoundB, uppBoundB ] = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax )

    %check special case 
    if GnOffset == 0 && I_rest == 0
        lowBoundA   = t_const;
        uppBoundA   = Inf;
        lowBoundB   = NaN;
        uppBoundB   = NaN;
    end

    %this method calculates the ranges of convex and concave shapes
    switch sign(GnOffset)
        case +1
            %================================================================
            % POSITIVE GnOffset
            %================================================================
            switch sign(I_rest)
                case +1
                    %------------------------------
                    %-------   I_rest > 0   -------
                    %------------------------------
                    %I_rest is positive: The fastest accumulation is given by
                    %adding a concave shape to the linear phase. The slowest
                    %accumulation depends on whether the integral of the linear
                    %phase minus a triangular shape reaching the zero gradient
                    %line is larger than I_rest. If it is so, the time range is
                    %divided into two sub-ranges, including G_end being in the
                    %negative range. otherwise the range is continous.
                    
                    %get fastest accumulation; check integral at +Gmax constraint
                    t_raise         = (Gmax-GnOffset)/Smax;
                    I_switch        = t_raise*(Gmax+GnOffset);
                    
                    %get lower bound
                    %---------------
                    if      I_rest <= I_switch
                        %   use triangular shape positive
                        %   -(2*(GnOffset + (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        %   -(2*(GnOffset - (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA  	= t_const+max([t_rest_a,t_rest_b]);
                        
                    elseif  I_rest > I_switch
                        %use trapezoidal shape positive
                        %   (I_rest + t_raise*(Gmax - GnOffset))/Gmax
                        
                        t_rest     	= (I_rest + t_raise*(Gmax - GnOffset))/Gmax;
                        lowBoundA  	= t_const+t_rest;
                        
                    end
                    
                    %get upper bound
                    %---------------
                    
                    %decide how to reach slow accumulation.
                    t_zero  = GnOffset/Smax;
                    I_zero  = GnOffset*t_zero;
                    
                    %if the integral accumulated within reaching G = 0 is
                    %smaller I_rest, the upper t bound is Inf. Otherwise we
                    %need to find the last time point in the positive part and
                    %the first time point in the negative allowing the correct
                    %accumulation
                    if I_rest >= I_zero
                        %only one infitine bound
                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= Inf;
                        
                        %no subdivided range
                        lowBoundB   = NaN;
                        uppBoundB   = NaN;
                    else
                        %create subdivided bounds
                        t_zero_a = (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_zero_b = (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;

                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= t_const+t_zero_a;
                        lowBoundB 	= t_const+t_zero_b;
                        uppBoundB  	= Inf;
                    end
                    
                case {-1,0}
                    %------------------------------
                    %-------   I_rest < 0   -------
                    %------------------------------
                    %I_rest is negative. In this case the upper bound is Inf in
                    %any case.
                    uppBoundA = Inf;
                    
                    %get time point and integral, at which -Gmax constraint is
                    %reached
                    t_fall  	= (Gmax+GnOffset)/Smax;
                    I_switch 	= -t_fall*(Gmax-GnOffset);
                    
                    %get lower bounds
                    %------------------
                    if      I_rest >= I_switch
                        %use triangular shape negative
                        %   (2*(GnOffset + (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        %   (2*(GnOffset - (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA  	= t_const+max([t_rest_a,t_rest_b]);
                        uppBoundA  	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                    elseif  I_rest < I_switch
                        %use trapezoidal shape positive
                        %   -(I_rest - t_fall*(Gmax + GnOffset))/Gmax
                        
                        t_rest      = -(I_rest - t_fall*(Gmax + GnOffset))/Gmax;
                        lowBoundA	= t_rest + t_const;
                        uppBoundA 	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                    end
                    
            end
            
        case {-1,0}
            %================================================================
            % NEGATIVE GnOffset
            %================================================================
            switch sign(I_rest)
                case {+1,0}
                    %------------------------------
                    %-------   I_rest > 0   -------
                    %------------------------------
                    %I_rest is positive. In this case the upper bound is Inf in
                    %any case.
                    lowBoundA   = Inf;
                    
                    %get time point and integral, at which +Gmax constraint is
                    %reached
                    t_raise   	= (Gmax-GnOffset)/Smax;
                    I_switch 	= t_raise*(Gmax+GnOffset);
                    
                    %check minus bounds
                    %------------------
                    if      I_rest <= I_switch
                        %use triangular shape negative
                        %   -(2*(GnOffset + (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        %   -(2*(GnOffset - (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA 	= t_const+max([t_rest_a,t_rest_b]);
                        uppBoundA  	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                    elseif  I_rest > I_switch
                        %use trapezoidal shape positive
                        %   (I_rest + t_raise*(Gmax - GnOffset))/Gmax
                        
                        t_rest    	= (I_rest + t_raise*(Gmax - GnOffset))/Gmax;
                        lowBoundA	= t_rest + t_const;
                        uppBoundA  	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                    end
                    
                case -1
                    %------------------------------
                    %-------   I_rest < 0   -------
                    %------------------------------
                    %I_rest is negative: The fastest accumulation is given by
                    %subtracting a convex shape to the linear phase. The slowest
                    %accumulation depends on whether the integral of the linear
                    %phase plus a triangular shape reaching the zero gradient
                    %line is smaller than I_rest. If it is so, the time range is
                    %divided into two sub-ranges, including G_end being in the
                    %positive range. otherwise the range is continous.
                    
                    %get fastest accumulation; check integral at -Gmax constraint
                    t_fall   	= (Gmax+GnOffset)/Smax;
                    I_switch 	= -t_fall*(Gmax-GnOffset);
                    
                    %get lower bounds
                    %-----------------
                    if      I_rest >= I_switch
                        %use triangular shape positive
                        %   (2*(GnOffset + (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        %   (2*(GnOffset - (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA 	= t_const+max([t_rest_a,t_rest_b]);
                        
                    elseif  I_rest < I_switch
                        %use trapezoidal shape negative
                        %   (I_rest + t_fall*(Gmax - GnOffset))/Gmax
                        
                        t_rest    	= -(I_rest - t_fall*(Gmax + GnOffset))/Gmax;
                        lowBoundA 	= t_const+t_rest;
                        
                    end
                    
                    
                    %decide how to reach slow accumulation.
                    t_zero  = -GnOffset/Smax;
                    I_zero  = GnOffset*t_zero;
                    
                    %if the integral accumulated within reaching G = 0 is
                    %smaller I_rest, the upper t bound is Inf. Otherwise we
                    %need to find the last time point in the positive part and
                    %the first time point in the negative allowing the correct
                    %accumulation
                    if I_rest <= I_zero
                        %only one infitine bound
                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        
                    else
                        %create subdivided bounds
                        t_zero_a = -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_zero_b = -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;

                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA	= t_zero_a + t_const;
                        lowBoundB 	= t_const+t_zero_b;
                        uppBoundB 	= Inf;
                    end
                    
            end
            
    end
    
    if lowBoundA < 0
     	error('lowBoundA < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    if uppBoundA < 0
     	error('uppBoundA < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    if lowBoundB < 0
     	error('lowBoundB < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    if uppBoundB < 0
     	error('uppBoundB < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    
end

function [ lowBounds, uppBounds ] = MakeRangeContinous( lowBounds, uppBounds )

    %this method converts the given range in lowBounds and uppBounds to a
    %continous range by dropping the ranges in the beginning, that are not
    %continous. WARNING: this might lead to solutions, which are
    %suboptimal, where on the other hand, the convergence of the convex
    %optimzation might be influenced positively
    
    if size(lowBounds,1) == 1
        return;
    end

    %compare lower Bounds to upper Bounds
    ConInd = double( find( abs(lowBounds(2:end,1) - uppBounds(1:end-1,1)) > 0.001 , 1, 'last') );
    if ~isempty(ConInd)
        %delete ranges that make the full range incontinous
        lowBounds(1 : ConInd(1),:) = [];
        uppBounds(1 : ConInd(1),:) = [];
    end

end
