function [tlim,Gx,Gy,Gz,Sx,Sy,Sz,tx,ty,tz] = FullTimeOptGrad3D( GxIn, GxOut, GyIn, GyOut, GzIn, GzOut, dkx, dky, dkz, Gmax, Smax )

    %this method calls the GetValidTimeRanges method twices for both
    %dimensions and then determines tlim. In addition (to what is returned
    %by the the FastTimeOptGrad methods within the optimization), 
    %it returns the solved optimal gradients in x and y direction, i.e.
    %
    % - The switching times t
    % - The gradients
    % - The slew rates
    %
    %to establish the actual shape of the gradient waveform. 
        
    %use separate constraints in x and y (only important for RBF reformulation)
    if numel(Gmax) == 3
        Gmax_x  = Gmax(1);
        Gmax_y  = Gmax(2);
        Gmax_z  = Gmax(3);
    elseif numel(Gmax) == 1
        Gmax_x  = Gmax;
        Gmax_y  = Gmax;
        Gmax_z  = Gmax;
    else
        error('Gmax should have either one or three elements!')
    end
    
    if numel(Smax) == 3
        Smax_x  = Smax(1);
        Smax_y  = Smax(2);
        Smax_z  = Smax(3);
    elseif numel(Smax) == 1
        Smax_x  = Smax;
        Smax_y  = Smax;
        Smax_z  = Smax;
    else
        error('Smax should have either one or three elements!')
    end
    
    if  numel(GxIn) ~= 1 || numel(GxOut) ~= 1 || ...
        numel(GyIn) ~= 1 || numel(GyOut) ~= 1 || ...
        numel(GzIn) ~= 1 || numel(GzOut) ~= 1 || ...
        numel(dkx) ~= 1 || numel(dky) ~= 1 || numel(dkz) ~= 1
            error('Wrong input format!')
    end

    %check for correct input values
    if abs(GxIn) > Gmax_x
        GxIn = Gmax_x * sign(GxIn);
    end
    if abs(GyIn) > Gmax_y
        GyIn = Gmax_y * sign(GyIn);
    end
    if abs(GzIn) > Gmax_z
        GzIn = Gmax_z * sign(GzIn);
    end
    
    if abs(GxOut) > Gmax_x
        GxOut = Gmax_x * sign(GxOut);
    end
    if abs(GyOut) > Gmax_y
        GyOut = Gmax_y * sign(GyOut);
    end
    if abs(GzOut) > Gmax_z
        GzOut = Gmax_z * sign(GzOut);
    end

    %valid time ranges in X
    [ lowBoundsX, uppBoundsX, solIdx_X ] = GetValidTimeRanges( GxIn, GxOut, dkx, Gmax_x, Smax_x );
    %valid time ranges in Y
    [ lowBoundsY, uppBoundsY, solIdx_Y  ] = GetValidTimeRanges( GyIn, GyOut, dky, Gmax_y, Smax_y );
    %valid time ranges in Z
    [ lowBoundsZ, uppBoundsZ, solIdx_Z  ] = GetValidTimeRanges( GzIn, GzOut, dkz, Gmax_z, Smax_z );

    if isempty(lowBoundsX) || isempty(uppBoundsX)
        error('Empty X-bounds occured: \n GxIn  = %.4f \n GxOut = %.4f \n dkx   = %.4f \n Gmax  = %.4f \n Smax  = %.4f \n', ...
            GxIn, GxOut, dkx, Gmax_x, Smax_x) 
    end
    
    if isempty(lowBoundsY) || isempty(uppBoundsY)
        error('Empty Y-bounds occured: \n GyIn  = %.4f \n GyOut = %.4f \n dky   = %.4f \n Gmax  = %.4f \n Smax  = %.4f \n', ...
            GyIn, GyOut, dky, Gmax_y, Smax_y)  
    end
    
    if isempty(lowBoundsZ) || isempty(uppBoundsZ)
        error('Empty Z-bounds occured: \n GzIn  = %.4f \n GzOut = %.4f \n dkz   = %.4f \n Gmax  = %.4f \n Smax  = %.4f \n', ...
            GzIn, GzOut, dkz, Gmax_z, Smax_z)  
    end
    
    %get tlim in 3D
    [tlim, tmin_solIndex_A, tmin_solIndex_B, tmin_solIndex_C] = GetMinimumTime3D(   lowBoundsX, uppBoundsX, solIdx_X, ...
                                                                                    lowBoundsY, uppBoundsY, solIdx_Y, ...
                                                                                    lowBoundsZ, uppBoundsZ, solIdx_Z );
                                                                                
    %get correct solutions
    [ tx, Gx, Sx ] = GetValidTimeOptSolution( GxIn, GxOut, dkx, tlim, Smax_x, Gmax_x, tmin_solIndex_A );
    [ ty, Gy, Sy ] = GetValidTimeOptSolution( GyIn, GyOut, dky, tlim, Smax_y, Gmax_y, tmin_solIndex_B );
    [ tz, Gz, Sz ] = GetValidTimeOptSolution( GzIn, GzOut, dkz, tlim, Smax_z, Gmax_z, tmin_solIndex_C );

end


function [ C_lowBounds, C_uppBounds, C_solIndex ] = GetValidTimeRanges( GnIn, GnOut, dkn, Gmax, Smax )
  
    %error handling
    if abs(GnIn) > Gmax
        error('GnIn exceeds Gmax constraint')
    end
    
    if abs(GnOut) > Gmax
        error('GnOut exceeds Gmax constraint')
    end

    %warning('Gamma changed!')
    %Gamma       = 2.675222005e+2;
    Gamma   = 42.5775;
    %Gamma = 1;
    dknGamma    = dkn/Gamma;

    %final ranges
    %up to 5 overall ranges might be given for the solution, 2 ranges for the
    %fast convex or concave shape, 1 for the linear shape and 2 for the slow
    %convex or concave shape

    R1_lowBound     = NaN;
    R1_uppBound     = NaN;
    R1_solIndex     = NaN;
    
    R2_lowBound     = NaN;
    R2_uppBound     = NaN;
    R2_solIndex     = NaN;
    
    R3_lowBound     = NaN;
    R3_uppBound     = NaN;
    R3_solIndex     = NaN;
    
    R4_lowBound     = NaN;
    R4_uppBound     = NaN;
    R4_solIndex     = NaN;
    
    R5_lowBound     = NaN;
    R5_uppBound     = NaN;
    R5_solIndex     = NaN;

    t_const     = abs(GnIn - GnOut)/Smax;
    I_const 	= sign(GnIn - GnOut)*(GnIn^2 - GnOut^2)/(2*Smax);
    I_rest      = dknGamma - I_const;
    
    %special case, GnIn =0, GnOut = 0, dkn = 0
    if GnIn == 0 && GnOut == 0 && dkn == 0
        C_lowBounds     = 0;
        C_uppBounds     = Inf;
        C_solIndex      = 1;
        return
    end
    
    %get adjusted sign parameter
  	AdjustedSign = sign(GnIn - GnOut);
    if AdjustedSign == 0
        %in case AdjustedSign == 0, i.e. GnIn == GnOut, we have to decide, 
        %as how to treat the problem 
        switch sign(I_rest)
            case +1
                AdjustedSign = -1; 
            case -1
                AdjustedSign = +1;
            case 0
                AdjustedSign = +1; 
        end
        
    end

    %constant integral given be the connection of GnIn and GnOut
    switch AdjustedSign
        case -1
            %GnIn <= GnOut --> Increasing Shape

            %in the GnIn <= GnOut, the problem cannot be solved by the mono.
            %decreasing gradient type
            F2_lowBound     = NaN;
            F2_uppBound     = NaN;

            %depending on the signs of GnIn and GnOut, the range of the mono.
            %increasing gradient type is defined
            GnIn_Bound      = I_rest / GnIn;
            GnOut_Bound     = I_rest / GnOut;
            
            if isnan(GnIn_Bound)
                GnIn_Bound = 0.0;
            end
            if isnan(GnOut_Bound)
                GnOut_Bound = 0.0;
            end

            if      sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == +1
                F1_lowBound     = min([GnIn_Bound, GnOut_Bound]) + t_const;
                F1_uppBound     = max([GnIn_Bound, GnOut_Bound]) + t_const;

            elseif  sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == -1
                F1_lowBound     = GnIn_Bound + t_const;
                F1_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == +1
                F1_lowBound     = GnOut_Bound + t_const;
                F1_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == -1
                F1_lowBound     = NaN;
                F1_uppBound     = NaN;

            elseif  I_rest == 0
                if sign(GnIn) < sign(GnOut) || sign(GnIn) > sign(GnOut)
                    F1_lowBound     = t_const;
                    F1_uppBound     = Inf;
                else
                    F1_lowBound     = t_const;
                    F1_uppBound     = t_const;
                end
            else
                error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
            end

        case +1
            %GnIn >= GnOut --> Decreasing Shape
            I_rest   = dknGamma - I_const;

            %in the GnIn <= GnOut, the problem cannot be solved by the mono.
            %decreasing gradient type
            F1_lowBound     = NaN;
            F1_uppBound     = NaN;

            %depending on the signs of GnIn and GnOut, the range of the mono.
            %increasing gradient type is defined
            GnIn_Bound      = I_rest / GnIn;
            GnOut_Bound     = I_rest / GnOut;

            if      sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == +1
                F2_lowBound     = min([GnIn_Bound, GnOut_Bound]) + t_const;
                F2_uppBound     = max([GnIn_Bound, GnOut_Bound]) + t_const;

            elseif  sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == -1
                F2_lowBound     = GnIn_Bound + t_const;
                F2_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == +1
                F2_lowBound     = GnOut_Bound + t_const;
                F2_uppBound     = Inf;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == -1
                F2_lowBound     = NaN;
                F2_uppBound     = NaN;

            elseif  I_rest == 0
                if sign(GnIn) < sign(GnOut) || sign(GnIn) > sign(GnOut)
                    F2_lowBound     = t_const;
                    F2_uppBound     = Inf;
                else
                    F2_lowBound     = t_const;
                    F2_uppBound     = t_const;
                end
            else
            	error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)

            end

        otherwise
            error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
    end

    %fprintf('F1-Bounds (increasing):   [%.4f,...,%.4f]\n',F1_lowBound,F1_uppBound)
    %fprintf('F2-Bounds (decreasing):   [%.4f,...,%.4f]\n',F2_lowBound,F2_uppBound)

    %Get concave and convex parameters by subfunction
    switch AdjustedSign
        case -1
            %GnIn <= GnOut --> Increasing Shape
            switch sign(I_rest)
                case -1

                    % FAST:     F3 Convex shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F4 Concave shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F3_lowBoundA;
                    R1_uppBound     = F3_uppBoundA;
                    R1_solIndex     = 3;
                    
                    R2_lowBound     = F3_lowBoundB;
                    R2_uppBound     = F3_uppBoundB;
                    R2_solIndex     = 3;

                    %linear
                    R3_lowBound     = F1_lowBound;
                    R3_uppBound     = F1_uppBound;
                    R3_solIndex     = 1;

                    %slow
                    R4_lowBound     = F4_lowBoundA;
                    R4_uppBound     = F4_uppBoundA;
                    R4_solIndex     = 4;
                    
                    R5_lowBound     = F4_lowBoundB;
                    R5_uppBound     = F4_uppBoundB;
                    R5_solIndex     = 4;

                case +1

                    % FAST:     F4 Concave shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F3 Convex shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F4_lowBoundA;
                    R1_uppBound     = F4_uppBoundA;
                    R1_solIndex     = 4;
                    
                    R2_lowBound     = F4_lowBoundB;
                    R2_uppBound     = F4_uppBoundB;
                    R2_solIndex     = 4;

                    %linear
                    R3_lowBound     = F1_lowBound;
                    R3_uppBound     = F1_uppBound;
                    R3_solIndex     = 1;

                    %slow
                    R4_lowBound     = F3_lowBoundA;
                    R4_uppBound     = F3_uppBoundA;
                    R4_solIndex     = 3;
                    
                    R5_lowBound     = F3_lowBoundB;
                    R5_uppBound     = F3_uppBoundB;
                    R5_solIndex     = 3;
                    
           	case 0
                %no residual integration; I_rest = 0
                    if (sign(GnIn) == -1 && sign(GnOut) == +1) || ...
                       (sign(GnIn) == +1 && sign(GnOut) == -1)  
                            %set full increasing
                            R1_lowBound     = F1_lowBound;
                            R1_uppBound     = F1_uppBound;
                            R1_solIndex     = 1;
                    else
                    
                        if abs(GnIn) <= abs(GnOut)
                            GnOffset    = GnIn;
                        else
                            GnOffset    = GnOut;
                        end

                        %Depending on the sign of GnOffset, a convex or concave
                        %shape will be used

                        [ F34_lowBoundA, F34_uppBoundA, ~, ~ ] ...
                            = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax );

                        %no faster solution than increasing possible
                        R1_lowBound     = NaN;
                        R1_uppBound     = NaN;
                        R1_solIndex     = NaN;
                        
                        R2_lowBound     = NaN;
                        R2_uppBound     = NaN;
                        R2_solIndex     = NaN;

                        R3_lowBound     = F1_lowBound;
                        R3_uppBound     = F1_uppBound;
                        R3_solIndex     = 1;

                        R4_lowBound     = F34_lowBoundA;
                        R4_uppBound     = F34_uppBoundA;

                        R5_lowBound     = NaN;
                        R5_uppBound     = NaN;
                        R5_solIndex     = NaN;

                        %Set basis function identifier
                        switch sign(GnOffset)
                            case -1
                                %Use Concave, if GnOffset < 0
                                R4_solIndex     = 4;
                            case +1
                                %Use Convex, if GnOffset > 0
                                R4_solIndex     = 3;
                            case 0
                                %Unsolvable, if GnOffset == 0
                                %R3_solIndex     = 1;
                                R4_solIndex     = NaN;
                        end
                    
                    end
            end

        case +1
            %GnIn >= GnOut --> Decreasing Shape
            switch sign(I_rest)
                case -1
                    % FAST:     F4 Convex shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F3 Concave shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F4_lowBoundA;
                    R1_uppBound     = F4_uppBoundA;
                    R1_solIndex     = 3;
                    
                    R2_lowBound     = F4_lowBoundB;
                    R2_uppBound     = F4_uppBoundB;
                    R2_solIndex     = 3;

                    %linear
                    R3_lowBound     = F2_lowBound;
                    R3_uppBound     = F2_uppBound;
                    R3_solIndex     = 2;

                    %slow
                    R4_lowBound     = F3_lowBoundA;
                    R4_uppBound     = F3_uppBoundA;
                    R4_solIndex     = 4;
                    
                    R5_lowBound     = F3_lowBoundB;
                    R5_uppBound     = F3_uppBoundB;
                    R5_solIndex     = 4;

                case +1

                    % FAST:     F3 Convex shape at the beginning
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                        = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );

                    % SLOW:     F4 Concave shape at the end
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                        ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );

                    %fast
                    R1_lowBound     = F3_lowBoundA;
                    R1_uppBound     = F3_uppBoundA;
                    R1_solIndex     = 4;
                    
                    R2_lowBound     = F3_lowBoundB;
                    R2_uppBound     = F3_uppBoundB;
                    R2_solIndex     = 4;

                    %linear
                    R3_lowBound     = F2_lowBound;
                    R3_uppBound     = F2_uppBound;
                    R3_solIndex     = 2;

                    %slow
                    R4_lowBound     = F4_lowBoundA;
                    R4_uppBound     = F4_uppBoundA;
                    R4_solIndex     = 3;
                    
                    R5_lowBound     = F4_lowBoundB;
                    R5_uppBound     = F4_uppBoundB;
                    R5_solIndex     = 3;
                    
         	case 0
                %no residual integration; I_rest = 0
                    if (sign(GnIn) == -1 && sign(GnOut) == +1) || ...
                       (sign(GnIn) == +1 && sign(GnOut) == -1)  
                            %set full decreasing
                            R1_lowBound     = F2_lowBound;
                            R1_uppBound     = F2_uppBound;
                            R1_solIndex     = 2;
                    else
                    
                        if abs(GnIn) <= abs(GnOut)
                            GnOffset    = GnIn;
                        else
                            GnOffset    = GnOut;
                        end

                        %Depending on the sign of GnOffset, a convex or concave
                        %shape will be used

                        [ F34_lowBoundA, F34_uppBoundA, ~, ~ ] ...
                            = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax );

                        %no faster solution than increasing possible
                        R1_lowBound     = NaN;
                        R1_uppBound     = NaN;
                        R1_solIndex     = NaN;
                        
                        R2_lowBound     = NaN;
                        R2_uppBound     = NaN;
                        R2_solIndex     = NaN;

                        R3_lowBound     = F2_lowBound;
                        R3_uppBound     = F2_uppBound;
                        R3_solIndex     = 2;

                        R4_lowBound     = F34_lowBoundA;
                        R4_uppBound     = F34_uppBoundA;

                        R5_lowBound     = NaN;
                        R5_uppBound     = NaN;
                        R5_solIndex     = NaN;

                        %Set basis function identifier
                        switch sign(GnOffset)
                            case -1
                                %Use Concave, if GnOffset < 0
                                R4_solIndex     = 4;
                            case +1
                                %Use Convex, if GnOffset > 0
                                R4_solIndex     = 3;
                            case 0
                                %Unsolvable, if GnOffset == 0
                                %R3_solIndex     = 1;
                                R4_solIndex     = NaN;
                        end
                    
                    end     
            end
    end

    %Get concatenated ranges
    C_lowBounds     = nan(5,1);
    C_uppBounds     = nan(5,1);
    C_solIndex      = nan(5,1);

    while(1)

        %Set low limits of part 1
        C_lowBounds(1) = R1_lowBound;
        C_uppBounds(1) = R1_uppBound;
        C_solIndex(1)  = R1_solIndex;

        %return, if upper bound is inf
        if isinf(R1_uppBound) && isnan(R3_lowBound) || ...
           isinf(R1_uppBound) && isinf(R3_lowBound)     
            break;
        end

        if      R1_uppBound >= R3_lowBound
            C_uppBounds(1) = R3_lowBound;
        elseif  R2_lowBound >= R3_lowBound
            C_lowBounds(2) = R2_lowBound;
            C_uppBounds(2) = R2_uppBound;
            C_solIndex(2)  = R2_solIndex;
            %again, adjust upper
            if R2_uppBound >= R3_lowBound
                C_uppBounds(2) = R3_lowBound;
            end
        end
        
        if isinf(R2_uppBound) && isnan(R3_lowBound) && isnan(R3_uppBound)
            C_lowBounds(2) = R2_lowBound;
            C_uppBounds(2) = R2_uppBound;
            C_solIndex(2)  = R2_solIndex;
            break;
        end

        if isnan(R3_lowBound) && isnan(R3_uppBound)
           break; 
        end
        
        %set linear phase bounds
        C_lowBounds(3) = R3_lowBound;
        C_uppBounds(3) = R3_uppBound;
        C_solIndex(3)  = R3_solIndex;

        %return, if upper bound is inf
        if isinf(R3_uppBound)
            break;
        end 

        %Set low limits of part 3
        C_lowBounds(4) = R4_lowBound;
        C_uppBounds(4) = R4_uppBound;
        C_solIndex(4)  = R4_solIndex;

        %adjust to linear bounds
        if R4_lowBound < R3_uppBound
            C_lowBounds(4) = R3_uppBound;
        end

        %return, if upper bound is inf
        if isinf(R4_uppBound)
            break;
        end 

        %Set low limits of part 3
        C_lowBounds(5) = R5_lowBound;
        C_uppBounds(5) = R5_uppBound;
        C_solIndex(5)  = R5_solIndex;

        %return, if upper bound is inf
        if isinf(R5_uppBound)
            break;
        end 

        error('Limited time soltution occurred!')

    end

    %unequal nans in time ranges
    if min(isnan(C_lowBounds) == isnan(C_uppBounds) == 0)
        error('NaN occurred in time ranges!')
    end

    C_lowBounds(isnan(C_lowBounds)) = [];
    C_uppBounds(isnan(C_uppBounds)) = [];
    C_solIndex(isnan(C_solIndex))   = [];

end
    
    
%Subfunction to deal with the convex and concave parts
function [ lowBoundA, uppBoundA, lowBoundB, uppBoundB ] = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax )

    %check special case 
    if GnOffset == 0 && I_rest == 0
        lowBoundA   = t_const;
        uppBoundA   = Inf;
        lowBoundB   = NaN;
        uppBoundB   = NaN;
    end

    %this method calculates the ranges of convex and concave shapes
    switch sign(GnOffset)
        case +1
            %================================================================
            % POSITIVE GnOffset
            %================================================================
            switch sign(I_rest)
                case +1
                    %------------------------------
                    %-------   I_rest > 0   -------
                    %------------------------------
                    %I_rest is positive: The fastest accumulation is given by
                    %adding a concave shape to the linear phase. The slowest
                    %accumulation depends on whether the integral of the linear
                    %phase minus a triangular shape reaching the zero gradient
                    %line is larger than I_rest. If it is so, the time range is
                    %divided into two sub-ranges, including G_end being in the
                    %negative range. otherwise the range is continous.
                    
                    %get fastest accumulation; check integral at +Gmax constraint
                    t_raise         = (Gmax-GnOffset)/Smax;
                    I_switch        = t_raise*(Gmax+GnOffset);
                    
                    %get lower bound
                    %---------------
                    if      I_rest <= I_switch
                        %   use triangular shape positive
                        %   -(2*(GnOffset + (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        %   -(2*(GnOffset - (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA  	= t_const+max([t_rest_a,t_rest_b]);
                        
                    elseif  I_rest > I_switch
                        %use trapezoidal shape positive
                        %   (I_rest + t_raise*(Gmax - GnOffset))/Gmax
                        
                        t_rest     	= (I_rest+t_raise*(Gmax-GnOffset))/Gmax;
                        lowBoundA  	= t_const+t_rest;
                        
                    end
                    
                    %get upper bound
                    %---------------
                    
                    %decide how to reach slow accumulation.
                    t_zero  = GnOffset/Smax;
                    I_zero  = GnOffset*t_zero;
                    
                    %if the integral accumulated within reaching G = 0 is
                    %smaller I_rest, the upper t bound is Inf. Otherwise we
                    %need to find the last time point in the positive part and
                    %the first time point in the negative allowing the correct
                    %accumulation
                    if I_rest >= I_zero
                        %only one infitine bound
                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= Inf;
                        
                        %no subdivided range
                        lowBoundB   = NaN;
                        uppBoundB   = NaN;
                    else
                        %create subdivided bounds
                        t_zero_a = (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_zero_b = (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;

                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= t_const+t_zero_a;
                        lowBoundB 	= t_const+t_zero_b;
                        uppBoundB  	= Inf;
                    end
                    
                case {-1,0}
                    %------------------------------
                    %-------   I_rest < 0   -------
                    %------------------------------
                    %I_rest is negative. In this case the upper bound is Inf in
                    %any case.
                    uppBoundA = Inf;
                    
                    %get time point and integral, at which -Gmax constraint is
                    %reached
                    t_fall  	= (Gmax+GnOffset)/Smax;
                    I_switch 	= -t_fall*(Gmax-GnOffset);
                    
                    %get lower bounds
                    %------------------
                    if      I_rest >= I_switch
                        %use triangular shape negative
                        %   (2*(GnOffset + (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        %   (2*(GnOffset - (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA  	= t_const+max([t_rest_a,t_rest_b]);
                        uppBoundA  	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                    elseif  I_rest < I_switch
                        %use trapezoidal shape positive
                        %   -(I_rest - t_fall*(Gmax + GnOffset))/Gmax
                        
                        t_rest      = -(I_rest-t_fall*(Gmax+GnOffset))/Gmax;
                        lowBoundA	= t_rest + t_const;
                        uppBoundA 	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                    end
                    
            end
            
        case {-1,0}
            %================================================================
            % NEGATIVE GnOffset
            %================================================================
            switch sign(I_rest)
                case {+1,0}
                    %------------------------------
                    %-------   I_rest > 0   -------
                    %------------------------------
                    %I_rest is positive. In this case the upper bound is Inf in
                    %any case.
                    lowBoundA   = Inf;
                    
                    %get time point and integral, at which +Gmax constraint is
                    %reached
                    t_raise   	= (Gmax-GnOffset)/Smax;
                    I_switch 	= t_raise*(Gmax+GnOffset);
                    
                    %check minus bounds
                    %------------------
                    if      I_rest <= I_switch
                        %use triangular shape negative
                        %   -(2*(GnOffset + (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        %   -(2*(GnOffset - (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        if t_rest_a < 0 && t_rest_b < 0
                           error('t_rest_a < 0 and t_rest_b < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
                        end
                        
                        lowBoundA 	= t_const+max([t_rest_a,t_rest_b]);
                        uppBoundA  	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                    elseif  I_rest > I_switch
                        %use trapezoidal shape positive
                        %   (I_rest + t_raise*(Gmax - GnOffset))/Gmax
                        
                        t_rest    	= (I_rest+t_raise*(Gmax-GnOffset))/Gmax;
                        lowBoundA	= t_rest + t_const;
                        uppBoundA  	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                    end
                    
                case -1
                    %------------------------------
                    %-------   I_rest < 0   -------
                    %------------------------------
                    %I_rest is negative: The fastest accumulation is given by
                    %subtracting a convex shape to the linear phase. The slowest
                    %accumulation depends on whether the integral of the linear
                    %phase plus a triangular shape reaching the zero gradient
                    %line is smaller than I_rest. If it is so, the time range is
                    %divided into two sub-ranges, including G_end being in the
                    %positive range. otherwise the range is continous.
                    
                    %get fastest accumulation; check integral at -Gmax constraint
                    t_fall   	= (Gmax+GnOffset)/Smax;
                    I_switch 	= -t_fall*(Gmax-GnOffset);
                    
                    %get lower bounds
                    %-----------------
                    if      I_rest >= I_switch
                        %use triangular shape positive
                        %   (2*(GnOffset + (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        %   (2*(GnOffset - (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        lowBoundA 	= t_const+max([t_rest_a,t_rest_b]);
                        
                    elseif  I_rest < I_switch
                        %use trapezoidal shape negative
                        %   (I_rest + t_fall*(Gmax - GnOffset))/Gmax
                        
                        t_rest    	= -(I_rest-t_fall*(Gmax+GnOffset))/Gmax;
                        lowBoundA 	= t_const+t_rest;
                        
                    end
                    
                    
                    %decide how to reach slow accumulation.
                    t_zero  = -GnOffset/Smax;
                    I_zero  = GnOffset*t_zero;
                    
                    %if the integral accumulated within reaching G = 0 is
                    %smaller I_rest, the upper t bound is Inf. Otherwise we
                    %need to find the last time point in the positive part and
                    %the first time point in the negative allowing the correct
                    %accumulation
                    if I_rest <= I_zero
                        %only one infitine bound
                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= Inf;
                        
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        
                    else
                        %create subdivided bounds
                        t_zero_a = -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_zero_b = -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;

                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA	= t_zero_a + t_const;
                        lowBoundB 	= t_const+t_zero_b;
                        uppBoundB 	= Inf;
                    end
                    
            end
    end
    
    
    if lowBoundA < 0
     	error('lowBoundA < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    if uppBoundA < 0
     	error('uppBoundA < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    if lowBoundB < 0
     	error('lowBoundB < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
    if uppBoundB < 0
     	error('uppBoundB < 0 \n GnOffset =  %.8f \n I_rest   = %.8f \n t_const  = %.8f \n', GnOffset, I_rest, t_const) 
    end
end

function [tlim, tmin_solIndex_A, tmin_solIndex_B, tmin_solIndex_C] = GetMinimumTime3D(  lowBoundsA, uppBoundsA, solIdx_A, ...
                                                                                        lowBoundsB, uppBoundsB, solIdx_B, ...
                                                                                        lowBoundsC, uppBoundsC, solIdx_C )

    %this method is used to determine the smallest tlim contained in both
    %ranges A and B, given by n time ranges in lowBoundsA and uppBoundsA
    %and m time ranges in lowBoundsB and uppBoundsB. The method should find
    %a tlim in any case, since each dimension contains a range with upper
    %bound being infinity
    
 	%predefintion
    InitIdx     = find( max([lowBoundsA(end,1),lowBoundsB(end,1),lowBoundsC(end,1)]) == max([lowBoundsA(end,1),lowBoundsB(end,1),lowBoundsC(end,1)]), 1, 'first');
    switch InitIdx(1)
        case 1
            tlim        = lowBoundsA(end,1);
        case 2
            tlim        = lowBoundsB(end,1);
        otherwise
            tlim        = lowBoundsC(end,1);
    end
    
    tmin_solIndex_A     = solIdx_A( end );
    tmin_solIndex_B     = solIdx_B( end );
    tmin_solIndex_C     = solIdx_C( end );
    
    while(1)
        
        %get largest of lowest bounds in A and B
        CurLim      = double( find(max([lowBoundsA(1),lowBoundsB(1),lowBoundsC(1)]) == [lowBoundsA(1),lowBoundsB(1),lowBoundsC(1)], 1) );
        
        switch CurLim(1)
            case 1
                %check, if lowest Bound of A is in B
                % YES: return
                %  NO: remove first range of B
                Bool = [max( lowBoundsA(1,1) <= uppBoundsB(:,1) & lowBoundsA(1,1) >= lowBoundsB(:,1) ) == 1, ...
                        max( lowBoundsA(1,1) <= uppBoundsC(:,1) & lowBoundsA(1,1) >= lowBoundsC(:,1) ) == 1 ];
                if all(Bool)
                    tlim    = lowBoundsA(1,1);
                    
                    %get correct solution identifiers
                    tmin_solIndex_A     = solIdx_A( 1 );
                    tmin_solIndex_B     = solIdx_B( find( lowBoundsA(1,1) <= uppBoundsB(:,1) & lowBoundsA(1,1) >= lowBoundsB(:,1),1,'first' ) );
                    tmin_solIndex_C     = solIdx_C( find( lowBoundsA(1,1) <= uppBoundsC(:,1) & lowBoundsA(1,1) >= lowBoundsC(:,1),1,'first' ) );
                    
                    return;
                else
                    %only delete the intervals which are not solvable
                    if ~Bool(1)
                        lowBoundsB(1,:)   = [];
                        uppBoundsB(1,:)   = [];
                        solIdx_B(1)       = [];
                    end
                    if ~Bool(2)
                        lowBoundsC(1,:)   = [];
                        uppBoundsC(1,:)   = [];
                        solIdx_C(1)       = [];
                    end
                    
                end
                
        	case 2
                %check, if lowest Bound of B is in A & C
                % YES: return
                %  NO: remove first range of A & C
                Bool = [max( lowBoundsB(1,1) <= uppBoundsA(:,1) & lowBoundsB(1,1) >= lowBoundsA(:,1) ) == 1,...
                        max( lowBoundsB(1,1) <= uppBoundsC(:,1) & lowBoundsB(1,1) >= lowBoundsC(:,1) ) == 1  ];
                if all(Bool)   
                    tlim        = lowBoundsB(1,1);
                    
                    %get correct solution identifiers
                    tmin_solIndex_A     = solIdx_A( find( lowBoundsB(1,1) <= uppBoundsA(:,1) & lowBoundsB(1,1) >= lowBoundsA(:,1),1,'first' ) );
                    tmin_solIndex_B     = solIdx_B( 1 );
                    tmin_solIndex_C     = solIdx_C( find( lowBoundsB(1,1) <= uppBoundsC(:,1) & lowBoundsB(1,1) >= lowBoundsC(:,1),1,'first' ) );
                    
                    return;
                else
                    if ~Bool(1)
                        lowBoundsA(1,:)   = [];
                        uppBoundsA(1,:)   = [];
                        solIdx_A(1)       = [];
                    end
                    if ~Bool(2)
                        lowBoundsC(1,:)   = [];
                        uppBoundsC(1,:)   = [];
                        solIdx_C(1)       = [];
                    end
                end
                
            case 3
                %check, if lowest Bound of C is in A & B
                % YES: return
                %  NO: remove first range of A & B
                Bool = [max( lowBoundsC(1,1) <= uppBoundsA(:,1) & lowBoundsC(1,1) >= lowBoundsA(:,1) ) == 1, ...
                        max( lowBoundsC(1,1) <= uppBoundsB(:,1) & lowBoundsC(1,1) >= lowBoundsB(:,1) ) == 1];
                if all(Bool)     
                    tlim        = lowBoundsC(1,1);
                    
                    %get correct solution identifiers
                    tmin_solIndex_A     = solIdx_A( find( lowBoundsC(1,1) <= uppBoundsA(:,1) & lowBoundsC(1,1) >= lowBoundsA(:,1),1,'first' ) );
                    tmin_solIndex_B     = solIdx_B( find( lowBoundsC(1,1) <= uppBoundsB(:,1) & lowBoundsC(1,1) >= lowBoundsB(:,1),1,'first' ) );
                    tmin_solIndex_C     = solIdx_C( 1 );
                    
                    return;
                else
                    if ~Bool(1)
                        lowBoundsA(1,:)   = [];
                        uppBoundsA(1,:)   = [];
                        solIdx_A(1)       = [];
                    end
                    if ~Bool(2)
                        lowBoundsB(1,:)   = [];
                        uppBoundsB(1,:)   = [];
                        solIdx_B(1)       = [];
                    end
                    
                end
        end

        if isempty(lowBoundsA) || isempty(uppBoundsA) || ...
           isempty(lowBoundsB) || isempty(uppBoundsB) || ...
           isempty(lowBoundsC) || isempty(uppBoundsC)
            break;
        end

    end 
    
end


function [ tn, Gn, Sn ] = GetValidTimeOptSolution( GnIn, GnOut, dkn, tlim, Smax, Gmax, solIndex )

    verbosity = 0;

    % Solution
    Gamma   = 42.5775;
    dknGamma    = dkn / Gamma;

    %error handling
    if isempty(solIndex)
       save('Temp.mat','solIndex','GnIn','GnOut','dkn','tlim','Smax','Gmax')
       error('No appropiate basis function found!')
    elseif numel(solIndex) > 1
       warning('WarnTests:convertTest','Multiple basis functions found')
       solIndex = solIndex(1);
    else
        %disp(['*** Solution Basis Function #',num2str(solIndex),' - ',TypeStrings{solIndex},' ***'])
    end

    t_const     = abs(GnIn-GnOut)/Smax;
    I_const 	= (sign(GnIn-GnOut)*(GnIn^2-GnOut^2)*(1.0/2.0))/Smax;
    
    I_rest      = -I_const+dknGamma;
    t_rest      = -t_const+tlim;
    
    switch solIndex
        case {1,2}
            % Use Increasing or Decreasing Basis Function
            %-------------------------------
            G_p     = I_rest/t_rest;
            tau1    = abs(G_p-GnIn)/Smax;
            tau2    = t_rest+tau1;
            
        case 3
            % Use Convex Basis Function
            %---------------------------
            
            %determine gradient offset, this is either GnIn of GnOut
          	GnOffset = min([GnIn, GnOut]);
           
            G_pa 	= real(sqrt(Smax)*sqrt(I_rest*4.0-GnOffset*t_rest*4.0+Smax*t_rest^2))*(1.0/2.0)-real(Smax*t_rest)*(1.0/2.0)+real(GnOffset);
            tau1a 	= abs(G_pa-GnIn)/Smax;
            tau2a  	= tlim-abs(G_pa-GnOut)/Smax;
            
            G_pb    = real(sqrt(Smax)*sqrt(I_rest*4.0-GnOffset*t_rest*4.0+Smax*t_rest^2))*(-1.0/2.0)-real(Smax*t_rest)*(1.0/2.0)+real(GnOffset);
            tau1b 	= abs(G_pb-GnIn)/Smax;
            tau2b  	= tlim-abs(G_pb-GnOut)/Smax;
            
            if      0 <= tau1a && tau1a <= tau2a && tau2a <= tlim
                G_p     = G_pa;
                tau1    = tau1a;
                tau2    = tau2a;
            else
                G_p     = G_pb;
                tau1    = tau1b;
                tau2    = tau2b;
            end
            
            
        case 4
            % Use Concave Basis Function
            %----------------------------
            
            %determine gradient offset, this is either GnIn of GnOut
          	GnOffset = max([GnIn, GnOut]);

            G_pa 	= real(sqrt(Smax)*sqrt(I_rest*-4.0+GnOffset*t_rest*4.0+Smax*t_rest^2))*(1.0/2.0)+real(Smax*t_rest)*(1.0/2.0)+real(GnOffset);
            tau1a 	= abs(G_pa-GnIn)/Smax;
            tau2a  	= tlim-abs(G_pa-GnOut)/Smax;
            
            G_pb    = real(sqrt(Smax)*sqrt(I_rest*-4.0+GnOffset*t_rest*4.0+Smax*t_rest^2))*(-1.0/2.0)+real(Smax*t_rest)*(1.0/2.0)+real(GnOffset);
            tau1b 	= abs(G_pb-GnIn)/Smax;
            tau2b  	= tlim-abs(G_pb-GnOut)/Smax;
            
            if      0 <= tau1a && tau1a <= tau2a && tau2a <= tlim
                G_p     = G_pa;
                tau1    = tau1a;
                tau2    = tau2a;
            else
                G_p     = G_pb;
                tau1    = tau1b;
                tau2    = tau2b;
            end
            
        otherwise
            error('Solution index undefined: %i',solIndex)
  
    end
    
    %adjust tau for numerical instabilites
    if tau1 < 0 && tau1 >= 1e-6
        if verbosity > 0
        	warning('WarnTests:convertTest','tau1 has been adjusted (tau1 = %.1e < 0.0)\n',tau1)
        end
        tau1 = 0.0;
    end
    if tau2 < tau1 && abs(tau1 - tau2) <= 1e-6
        if verbosity > 0
            warning('WarnTests:convertTest','tau1 & tau2 have been adjusted (tau1 = %.1e > %.1e = tau2; tau1-tau2 = %.2e)\n',tau1,tau2,tau1-tau2)
        end
        tau1 = mean([tau1,tau2]);
        tau2 = tau1;
    end
    tau1    = min([tau1,tlim]);
    tau2    = min([tau2,tlim]);
        
    %general solution structure
    if  ~isnan(G_p)
        Gn = [GnIn,     G_p,  	G_p,    GnOut];
        tn = [0.0,      tau1,  	tau2, 	tlim];
        
        %return correct solutions for slew rates. The slew rate at time ti
        %specifies the slew rate in the subsequent part [ti,...,ti+1]
        switch solIndex
            case 1
                %Strictly increasing:   +Smax/+Smax
                Sn = [ +Smax, 0.0, +Smax, 0.0 ];
            case 2
                %Strictly decreasing:   -Smax/-Smax
                Sn = [ -Smax, 0.0, -Smax, 0.0 ];
            case 3
                %Convex:                -Smax/+Smax
                Sn = [ -Smax, 0.0, +Smax, 0.0 ];
            case 4
                %Concav:                +Smax/-Smax
                Sn = [ +Smax, 0.0, -Smax, 0.0 ];
            otherwise
                error('Undefined solution index')   
        end
    else
        Gn = [GnIn, GnOut];
        tn = [0.0,  tlim];
        
        %return correct solutions for slew rates. The slew rate at time ti
        %specifies the slew rate in the subsequent part [ti,...,ti+1]
        switch solIndex
            case 1
                %Strictly increasing:   +Smax/+Smax
                Sn = [ +Smax, 0.0 ];
            case 2
                %Strictly decreasing:   -Smax/-Smax
                Sn = [ -Smax, 0.0 ];
            case 3
                %Convex:                -Smax/+Smax
                Sn = [ -Smax, 0.0 ];
            case 4
                %Concav:                +Smax/-Smax
                Sn = [ +Smax, 0.0 ];
            otherwise
                error('Undefined solution index')   
        end
    end
    
    %Verify integral
    I	= trapz(tn,Gn);
    
    %print intergral
    if verbosity > 0
        fprintf(' abs(I - dknGamma) = %.2e\n',abs(I - dknGamma))
    end
    
    if abs(I - dknGamma) > 1e-8
        
        fprintf('Desired integral = %.8f\n',dknGamma)
        fprintf('   Real integral = %.8f\n',I)
        
        figure
        hold on
        box on
        grid on

        line([0.0 tlim],[+Gmax +Gmax],'Linewidth',2,'Linestyle','--','Color','green')
        line([0.0 tlim],[-Gmax -Gmax],'Linewidth',2,'Linestyle','--','Color','green')

        %xlim([0.0, tlim])
        ylim([-1.1*Gmax, + 1.1*Gmax])


        plot( tn, Gn, 'Linewidth',1,'Color','blue' )


      	save('Temp.mat','solIndex','GnIn','GnOut','dkn','tlim','Smax','Gmax')
       	error('Wrong integration of gradient function')
    end

    %Final Error handling
    if  numel(tn) ~= numel(Gn) || ...
        max(isnan(tn)) > 0 || ...
        max(isnan(Gn)) > 0 || ...
        max( (abs(Gn) - Gmax) > 5e-3 ) || ...
        max( (tn(1:end-1) - tn(2:end)) > 1e-6 )
    
        disp(' ----- ERROR -----')
        fprintf('BaseFun    = %i\n',solIndex)
        fprintf('GnIn       = %.8f\n',GnIn)
        fprintf('GnOut      = %.8f\n',GnOut)
        fprintf('dkn        = %.8f\n',dkn)
        fprintf('tlim       = %.8f\n',tlim)
        fprintf('Smax       = %.8f\n',Smax)
        fprintf('Gmax       = %.8f\n',Gmax)
        fprintf('   t:     '),fprintf('%2.6f   ',tn),fprintf('\n')
        fprintf('G(t):     '),fprintf('%2.6f   ',Gn),fprintf('\n')
        
        save('Temp.mat','solIndex','GnIn','GnOut','dkn','tlim','Smax','Gmax')
        
        error('Final pulse doesnt satisfy requirements!')
    end
        
end
