function h = PlotControlPoints( T )

    %error handling
    if numel(T.kx) ~= numel(T.ky) || numel(T.kx) ~= numel(T.kz)
       error('kx, ky, and kz vectors must have same length!') 
    end
    
    %define type of trajectory 2D or 3D
    switch size( T.R, 2 )
        case 4
            DimType = 3;
        case 3
            DimType = 2;
    end
    
    if (numel(T.kx) ~= size(T.R,1) || size(T.R,2) ~= 4) && DimType == 3
       error('3D Slope constraint matrix has wrong dimension!') 
    end
    if (numel(T.kx) ~= size(T.R,1) || size(T.R,2) ~= 3) && DimType == 2
       error('2D Slope constraint matrix has wrong dimension!') 
    end

    %include first and last points, add NaN constraints
    kx  = T.kx;
    ky  = T.ky;
    kz  = T.kz;
    R   = T.R;
    
    %plot the kspace nodes to the current figure
    kmax        = max(abs([kx(:); ky(:); kz(:)]));
    ArrowLength	= 0.2 * kmax;
    
    h           = nan( 1,3 );
    FontWeight  = 'normal';
    FontSize    = 12;
    switch DimType
        
        case 3
            % ============================================================
            %   T H R E E   D I M E N S I O N A L   T R A J E C T O R Y
            % ============================================================
            for k = 1 : numel(kx)
                switch R(k,4)
                    case 3
                        %no slope constraint
                        %-------------------
                        h(1) = plot3(kx(k),ky(k),kz(k),'*','Marker','x','Markersize',7,'Linewidth',2,'Color','blue');
                        text(kx(k),ky(k),kz(k),sprintf('  %i',k),'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','blue')
                    case 2
                        %layer constraint
                        %----------------
                        plot3(kx(k),ky(k),kz(k),'-*','Marker','x','Markersize',7,'Linewidth',2,'Color',[1.0, 0.5490, 0.0])
                        text(kx(k),ky(k),kz(k),sprintf('  %i',k),'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color',[1.0, 0.5490, 0.0])
                        warning('Layer constraints are currently not supported, as they result in difficult control regions!')
                    case 1
                        %vector constraint
                        %-----------------
                        VNorm   = R(k,1:3)./norm(R(k,1:3));
                        line( [kx(k) + VNorm(1) * ArrowLength/2, kx(k) - VNorm(1) * ArrowLength/2], ...
                              [ky(k) + VNorm(2) * ArrowLength/2, ky(k) - VNorm(2) * ArrowLength/2], ...
                              [kz(k) + VNorm(3) * ArrowLength/2, kz(k) - VNorm(3) * ArrowLength/2],'Linewidth',2,'Color',[0 0.6 0])
                        h(2) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color',[0 0.6 0]);
                        text(kx(k),ky(k),kz(k),sprintf('  %i',k), 'FontSize',FontSize,'FontWeight',FontWeight,...
                                                'HorizontalAlignment','right', 'VerticalAlignment','bottom','Color',[0 0.6 0])
                    case 0
                        %fixed gradients
                        %---------------
                        h(3) = plot3(kx(k),ky(k),kz(k),'*','Marker','x','Markersize',7,'Linewidth',2,'Color','red');
                        text(kx(k),ky(k),kz(k),sprintf('  %i',k),'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','red')
                    otherwise
                        error('Undefined 3D-constraint ID detected (%i)!',R(k,4))
                end
            end
            
        case 2
            % =======================================================
            %   T W O   D I M E N S I O N A L   T R A J E C T O R Y
            % =======================================================
           for k = 1 : numel(kx)
                switch R(k,3)
                    case 2
                        %no slope constraint
                        %-------------------
                        h(1) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color','blue');
                        %text(kx(k),ky(k),kz(k),sprintf('  %i',k),'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','blue')
                    case 1
                        %vector constraint
                        %-----------------
                        VNorm   = R(k,1:2)./norm(R(k,1:2));
                        line( [kx(k) + VNorm(1) * ArrowLength/2, kx(k) - VNorm(1) * ArrowLength/2], ...
                              [ky(k) + VNorm(2) * ArrowLength/2, ky(k) - VNorm(2) * ArrowLength/2],'Linewidth',2,'Color',[0 0.6 0])
                        h(2) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color',[0 0.6 0]);
                        %text(kx(k),ky(k),kz(k),sprintf('  %i',k), 'FontSize',FontSize,'FontWeight',FontWeight,...
                        %                         'HorizontalAlignment','right', 'VerticalAlignment','bottom','Color',[0 0.6 0])
                    case 0
                        %fixed gradients
                        %---------------
                        h(3) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color','red');
                        %text(kx(k),ky(k),kz(k),sprintf('  %i',k),'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','red')
                    otherwise
                        error('Undefined 2D-constraint ID detected (%i)!',R(k,3))
                end
            end 
            
    end