function [ pulsedata, auxdata ] = PerformShapeOptimization( options )

    fprintf('=========================================================================\n')
    fprintf('====  I N I T I A L I Z I N G   S H A P E   O P T I M I Z A T I O N  ====\n')
    fprintf('=========================================================================\n')

    %add some 3rd party directories
    addpath('Ressources\')
    addpath('GradientBasisFunction_v1\')
    addpath('GradientBasisFunction_v1\mex')
    addpath('GradientBasisFunction_v1\solvers\minConf')
    addpath('GradientBasisFunction_v1\solvers\minFunc')
    addpath('GradientBasisFunction_v1\solvers\misc')
    addpath('GradientBasisFunction_v1\solvers\so11matl')
    
    %check options, add default values for missing fields
    options = CheckOptions( options );
    
    % some required global variables
    global DateString
    global CounterIter
    global CounterFunEval
    global LastFunValue
    global cur_alpha_power
    global run_alpha_update
    DateString          = [];
    LastFunValue        = [];
    CounterIter         = 0;
    CounterFunEval      = 0;
    cur_alpha_power     = options.alpha_power;
    run_alpha_update    = false;

    %parse options
    if ~options.optim_debugmode
        PrepareSaveFolder();
        diary('off')
        diary(sprintf('SavedIterations/%s/CommandOutput.txt',DateString))
        save(sprintf('SavedIterations/%s/options_called.mat',DateString), 'options');
        
        fprintf('\nInitialization\n')
        fprintf('--------------\n')
        fprintf('Iterations will be saved to ''SavedIterations/%s''!\n',DateString)
    else
        fprintf(2,'# # # # # # # # # # # # # # # # # # # # # # # # # # # #\n')
        fprintf(2,'# # #                                             # # #\n')
        fprintf(2,'# # #   W A R N I N G :   D E B U G   M O D E     # # #\n')
        fprintf(2,'# # #                                             # # #\n')
        fprintf(2,'# # # # # # # # # # # # # # # # # # # # # # # # # # # #\n')
        pause
        pulsedata   = [];
        auxdata     = [];
        return
    end

    [ options, B0, B1 ] = SetPulseOptions3D( options );
    
    %=========================================================
    %========   T A R G E T     D E F I N I T I O N   ========
    %=========================================================
    
    %perform convolution if required
    if options.blur_target > 0
        %get correct sigma in each dimension associated with the pixel size
        MinFiltSize = 5;
        Sigma_x     = options.blur_target / options.PixSiz_x;
        Sigma_y     = options.blur_target / options.PixSiz_y;
        Sigma_z     = options.blur_target / options.PixSiz_z;
        FiltSize    = max([ MinFiltSize, ...
                        ceil( 4 * max([ double( ~isinf(Sigma_x) ) * Sigma_x, ...
                                        double( ~isinf(Sigma_y) ) * Sigma_y, ...
                                        double( ~isinf(Sigma_z) ) * Sigma_z ])) ]);

        %design filter kernel in 3D
        if options.nz ~= 1
            ker_x       = fspecial( 'gaussian', [FiltSize, 1], Sigma_x );
            ker_y       = fspecial( 'gaussian', [FiltSize, 1], Sigma_y );
            ker_z       = fspecial( 'gaussian', [FiltSize, 1], Sigma_z );
            [K_x, K_y, K_z] = meshgrid( ker_x, ker_y, ker_z );
            Kernel      = K_x .* K_y .* K_z;

            options.targetflip_mat = convolution3D_FFTdomain( options.targetflip_mat, Kernel );
        else
            ker_x       = fspecial( 'gaussian', [FiltSize, 1], Sigma_x );
            ker_y       = fspecial( 'gaussian', [FiltSize, 1], Sigma_y );
            [K_x, K_y] = meshgrid( ker_x, ker_y );
            Kernel      = K_x .* K_y;

            options.targetflip_mat = imfilter( options.targetflip_mat, Kernel );
        end
    end
    options.targetflip_mat  = options.targetflip_mat .* options.roi;
    options.targetmagn_mat  = flip2magn( options.targetflip_mat );
    
    if options.verbosity > 1 || options.optim_debugmode
        display_nimages( magn2flip(options.targetmagn_mat) ); axis image off;
        colormap(hot); colorbar; title('Target -- FA [deg.]'), set(gcf,'name','Target (FA)')
        if any( ~isreal( options.targetflip_mat(:) ) )
            display_nimages( angle(options.targetflip_mat) ); axis image off;
            colormap(hot); colorbar; title('Target -- phase [rad.]'), set(gcf,'name','Target (Phase)')
        end
    end

    PrintTrajectoryInfo( options )
    fprintf('\n')
    fprintf('=========================================================================\n')
    fprintf('======     C O M P U T I N G   I N I T I A L   R F   P U L S E     ======\n')
    fprintf('=========================================================================\n')
    fprintf('\n')
    
    %get trajectory control points for current set of Phi, remove constraints
    [ kx_vec, ky_vec, kz_vec, R_mat ] = feval( options.trajectory_fun, options.phi_init );

    % get trajectory, perform hardware sampling
    NV = sum( R_mat(:,end) );
    G0 = zeros( NV,1 );

    fprintf('Computing initial Trajectory...');
    T_init         = PerformOptimization3D( kx_vec, ky_vec, kz_vec, R_mat, options.Gmax, options.Smax, G0, options.GBF );
    fprintf(' done (pulse duration = %f ms ).\n',T_init.t(end))
    T_init         = CalculateRFTrajectory( T_init, options );

    if options.verbosity > 1 || options.optim_debugmode
        PlotTrajectory( T_init ); set(gcf,'name','Initial k-space Trajectory')
        PlotGradients( T_init,'h' ); set(gcf,'name','Initial Gradient Waveform')
    end
    
    %prepare system matrix inputs
    fprintf('Desigining Initial RF pulse...\n')
    coords                          = [ options.xpos_all(:), options.ypos_all(:), options.zpos_all(:) ];
    [ k_mat, zeroCols, T_init ] 	= PrepareSystemMatrixInput( T_init, options );
    Ind                             = sub2ind([ options.nx, options.ny, options.nz ], options.xidx_all, options.yidx_all, options.zidx_all);
    b1mat                           = reshape( cell2mat(cellfun(@(x) x(Ind), B1, 'UniformOutput', false)), options.npixel, options.nchannel);
    b0vec                           = B0( Ind );

    % compute system matrix
    if options.enforceB0 ~= 0
        A = [ ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec - options.enforceB0 ); ...    
              ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec ); ...   
              ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec + options.enforceB0 ) ];
        targetmagn_mat = repmat( options.targetmagn_mat( Ind ), 3, 1 );
    else
        A =   ComputeSystemMatrix( coords, b1mat, options.grad_dt, options.sysmat_interp , k_mat, b0vec );            
        targetmagn_mat = options.targetmagn_mat( Ind );
    end

    A( :, zeroCols )    = 0.0;

    % Initial Pulse calculation
    %---------------------------

    switch options.optim_rfsolver
        case 1
            % Use Conjugate Gradient Solver
            %...............................
            
            if options.use_gpu == 1
                %use GPU to set up the matrices required in pcg
                try
                    A_gpu           	= gpuArray( A );
                    targetmagn_mat_gpu  = gpuArray( targetmagn_mat );
                    C                   = gather( A_gpu' * A_gpu );
                    d                   = gather( A_gpu' * targetmagn_mat_gpu );
                    clear A_gpu targetmagn_mat_gpu
                catch exception
                    % warning('WarnTests:convertTest','Cannot use GPU, not enough memory!')
                    % fprintf('Reason: %s\n', exception.message)
                    C  	= A' * A;
                    d  	= A' * targetmagn_mat;
                end
            else
                C       = A' * A;
                d       = A' * targetmagn_mat;
            end

            if options.optim_debugmode && 0
                % no search for regularizer
                RF_init = DesignPulse_ConjugateGradient( options, C, d );
            else
                % run search for regularization parameter
                RF_init = DesignPulse_ConjugateGradient( options, C, d, [], 10 );
            end
            
        case 2
            % Use Conjugate Gradient Solver
            %...............................
            
            RF_init = DesignPulse_LeastSquares( options, A, targetmagn_mat );
             
    end

    % compute forward magnetization and RMSE
    if options.enforceB0 ~= 0
        forwardmagn_init = CalculateMagnetization( options, A( options.npixel + 1 : 2 * options.npixel, : ), RF_init );
    else
        forwardmagn_init = CalculateMagnetization( options, A, RF_init );
    end

    [~, NRMSE_init] = ComputeForwardError( A, RF_init, targetmagn_mat );

    %free resources
    clear A

    if options.verbosity > 1 || options.optim_debugmode
        display_nimages( magn2flip(forwardmagn_init.mt) ); axis image off;
        colormap(hot); colorbar; title('Inital STA -- FA [deg.]'), set(gcf,'name','Inital STA (FA)')
    end

    if options.run_blochsim
        fprintf('Running Bloch Simulation of Initial RF Pulse...\n')
        RF_init_full = GetFullRF( RF_init, T_init );
        [blochmagn_mx, blochmagn_my, blochmagn_mz, blochmagn_mt, blochmagn_flip, blochmagn_phase ] ...
            = BlochSimulation(	options.npixel, options.nchannel, T_init.Pulse.t', [ T_init.Pulse.Gx', T_init.Pulse.Gy', T_init.Pulse.Gz' ], ...
                                [ options.xidx_all(:), options.yidx_all(:), options.zidx_all(:) ], coords, ...
                                [ options.nx, options.ny, options.nz ], b0vec, b1mat, ...
                                RF_init_full, options.blochsim_interp );

        blochmagn_init = struct('mx', blochmagn_mx, 'my',   blochmagn_my,   'mz',    blochmagn_mz, ...
                                'mt', blochmagn_mt, 'flip', blochmagn_flip, 'phase', blochmagn_phase );
        if ~options.optim_debugmode
            save(sprintf('SavedIterations/%s/init_blochmagn.mat',DateString), 'blochmagn_init');
        end

        if options.verbosity > 1 || options.optim_debugmode
            display_nimages( magn2flip(blochmagn_init.mt) ); axis image off;
            colormap(hot); colorbar; title('Inital Bloch Simulation -- FA [deg.]'), set(gcf,'name','Inital Bloch Simulation (FA)')
        end
    end

    %save initial data
    if ~options.optim_debugmode
        save(sprintf('SavedIterations/%s/init_data.mat',DateString), 'T_init','RF_init', 'NRMSE_init', 'forwardmagn_init', 'options');
    end
    
    fprintf('\n')
    fprintf('=========================================================================\n')
    fprintf('======   O P T I M I Z I N G    S H A P E    P A R A M E T E R S   ======\n')
    fprintf('=========================================================================\n')
    fprintf('\n')
    clear A C
    
    %  C O N S T R A I N T   S O L V E R
    %-------------------------------------
    fminconoptions = optimset(  'Display','none',...
                                'TolFun', options.optim_tolfun, ...
                                'TolCon',options.optim_tolcon, ...
                                'TolX',options.optim_tolx,...
                                'Algorithm','sqp', ...
                                'Diagnostics','off', ...
                                'MaxIter',options.optim_maxiter,...
                                'MaxFunEvals',options.optim_maxfunevals );

    % set finite differencing type
    switch options.optim_findifftype
        case 1
            % use Matlab's intern FinDiff routine
            fprintf('Using fmincon intern Finite Differencing...\n')
            f       = @( phi ) GetFunValueForPhi ( phi, options, b1mat, b0vec );
            fun     = @( phi ) EvaluateObjective( f, phi );

            %some specific options for fmincon
            fminconoptions.OutputFcn = { @PrintCurrentIteration, ...
                                         @PlotCurrentIteration, ...
                                         @SetIterationData };  
            fminconoptions.DiffMinChange   = 5e-2;
            fminconoptions.DiffMaxChange   = 1e-1;
            fminconoptions.GradObj         = 'off';
            fminconoptions.FinDiffRelStep  = 2e-2;
        case 2
            % use FinDiff routine within EstimateJacobian.m
            fprintf('Using user-defined Finite Differencing via ''EstimateJacobian.m''...\n')
            f       = @( phi, saveiterdata ) GetFunValueForPhi ( phi, options, b1mat, b0vec, saveiterdata );
            fun     = @( phi ) EstimateJacobian( f, phi, options );

            %some specific options for fmincon
            fminconoptions.OutputFcn       = { ... Command Printing is carried out in EstimateJacobian
                                        @PlotCurrentIteration, ...
                                        @SetIterationData };  
            fminconoptions.GradObj         = 'on';
            fminconoptions.Hessian         = 'lbfgs';
    end

    %set global variable of the file string
    fprintf('\n')
    fprintf('      Pulse Max Time:   %4.1f ms     (otherwise return +Inf fun. value)\n', 1e+3*options.pulse_maxtime)
    fprintf('  Pulse Penalty Time:   %4.1f ms \n', 1e+3*options.objfun_Tmax)
    fprintf('     Pulse Time Beta:   %4.1f ms \n', options.objfun_beta )
    fprintf('     System Max Size:   %4.1f GB     (otherwise return +Inf fun. value)\n', options.sysmat_maxsize / 1e+9)
    fprintf('              TolFun:   %.2e \n', options.optim_tolfun)
    fprintf('              TolCon:   %.2e \n', options.optim_tolcon)
    fprintf('                TolX:   %.2e \n', options.optim_tolx)
    fprintf('             MaxIter:   %i \n', options.optim_maxiter)
    fprintf('         MaxFunEvals:   %i \n', options.optim_maxfunevals)

    %save results as start values within the iteration
    %set inital objfundata struct
    objfundata.NRMSE    = NRMSE_init;
    objfundata.time     = T_init.t(end);
    objfundata.SAR      = NaN;
    objfundata.funValue = RegularizedObjectiveFunction( objfundata, options );
    
    if ~options.optim_debugmode
        T                   = T_init;
        RF                  = RF_init;
        phi_red             = options.phi_init( options.phi_isvariable > 0 );
        %The init_data is usually used by the function evaluation methods to give some start values to the
        %RF and trajectory computation. Espcially the RF method is accelerated if it gets some nice start
        %values (okay, now I saw that you saved it later)
        save(sprintf('SavedIterations/%s/iter_data.mat',DateString),'T','RF','objfundata','phi_red');

        %store some data for plotting
        forwardmagn = forwardmagn_init;
        save(sprintf('SavedIterations/%s/plot_data.mat',DateString),'T','RF','forwardmagn','options','objfundata');
    else
        fprintf(2,'# # # # # # # # # # # # # # # # # # # # # # # # # # # #\n')
        fprintf(2,'# # #                                             # # #\n')
        fprintf(2,'# # #    E N D    O F    D E B U G    M O D E     # # #\n')
        fprintf(2,'# # #                                             # # #\n')
        fprintf(2,'# # # # # # # # # # # # # # # # # # # # # # # # # # # #\n')
        pause
        pulsedata   = [];
        auxdata     = [];
        return;
    end
    
    fullruntime = tic;
    if options.optim_scalevariables == 1
        %scale down variables prior to calling fmincon
        [phi_opt_red,~,exitflag] = fmincon(fun, GetRelativeParameters( options.phi_init( options.phi_isvariable > 0 ), options.phi_bounds( options.phi_isvariable > 0, : ) ), [], [], [], [], zeros(sum(options.phi_isvariable > 0),1), ones(sum(options.phi_isvariable > 0),1), [],fminconoptions);
        phi_opt = options.phi_init;
        phi_opt( options.phi_isvariable > 0 ) = GetAbsoluteParameters( phi_opt_red, options.phi_bounds( options.phi_isvariable > 0, : ) );
    else
        %use original variables
        [phi_opt_red,~,exitflag] = fmincon(fun, phi( options.phi_isvariable > 0 ), [], [], [], [], options.phi_bounds( options.phi_isvariable > 0, 1 ), options.phi_bounds( options.phi_isvariable > 0, 2 ), [],fminconoptions);
        phi_opt = options.phi_init;
        phi_opt( options.phi_isvariable > 0 ) = phi_opt_red;
    end
    
    Runtime = toc(fullruntime);
 	PrintTerminationReason( exitflag );
    DeleteInvalidFiles()
    fprintf('Optimization terminated after %.1f minutes!\n',Runtime/60)
    fprintf('\n')
    
    diary('off');


    fprintf('=======================================================\n')
    fprintf('======          F I N A L I Z A T I O N          ======\n')
    fprintf('=======================================================\n')
    fprintf('\n')
    
    fprintf('Desigining Final RF pulse...\n')
    
    run_alpha_update = true;
    [objfundata_opt, T_opt, RF_opt, forwardmagn_opt] = GetFunValueForPhi ( phi_opt( options.phi_isvariable == 1 ), options, b1mat, b0vec );
    RF_opt_full 	= GetFullRF( RF_opt, T_opt );
    save(sprintf('SavedIterations/%s/final_RF_mat',DateString),'objfundata_opt', 'T_opt', 'phi_opt', 'RF_opt_full', 'forwardmagn_opt');

    %run bloch simulation
    if options.run_blochsim
        fprintf('Running Bloch Simulation of Final RF Pulse...\n')
        [blochmagn_mx, blochmagn_my, blochmagn_mz, blochmagn_mt, blochmagn_flip, blochmagn_phase ] ...
            = BlochSimulation( options.npixel, options.nchannel, T_opt.Pulse.t', [ T_opt.Pulse.Gx', T_opt.Pulse.Gy', T_opt.Pulse.Gz' ], ...
            [ options.xidx_all(:), options.yidx_all(:), options.zidx_all(:) ], coords, ...
            [ options.nx, options.ny, options.nz ], b0vec, b1mat, RF_opt_full, options.blochsim_interp );
        blochmagn_final     = struct( 'mx',  blochmagn_mx, 'my', blochmagn_my,'mz',  blochmagn_mz, 'mt',  blochmagn_mt, 'flip', blochmagn_flip, 'phase',  blochmagn_phase );
        auxdata.blochmagn   = blochmagn_final;
        save(sprintf('SavedIterations/%s/final_blochmagn.mat',DateString), 'blochmagn_final');
    else
        auxdata.blochmagn   = [];
    end

    %store all in AuxData
    auxdata.objfundata	= objfundata_opt;
    auxdata.T           = T_opt;
    auxdata.RF      	= RF_opt_full;
    auxdata.forwardmagn	= forwardmagn_opt;
    auxdata.options    	= options;

    ntimes         	= size(RF_opt_full,1) / options.nchannel;
    pulsedata.RF 	= reshape( RF_opt_full, ntimes, options.nchannel ).';
    pulsedata.Gx 	= T_opt.Pulse.Gx;
    pulsedata.Gy	= T_opt.Pulse.Gy;
    pulsedata.Gz	= T_opt.Pulse.Gz;
    pulsedata.t 	= T_opt.Pulse.t;

    %uses the RF waveform, and the raw data in T_Opt, only gradients
    %are interpolated, because this can be done accurately
    if options.RF_delay ~= 0
        [ Gx_delayed, Gy_delayed, Gz_delayed, RF_delayed, t_delayed ] = GetRFGradientDelay( pulsedata.RF, T_opt, options );
        pulsedata.RF_delayed 	= RF_delayed;
        pulsedata.Gx_delayed 	= Gx_delayed;
        pulsedata.Gy_delayed	= Gy_delayed;
        pulsedata.Gz_delayed	= Gz_delayed;
        pulsedata.t_delayed 	= t_delayed;
    end
    
    %save final data
    save(sprintf('SavedIterations/%s/final_pulsedata.mat',DateString),'pulsedata');
    save(sprintf('SavedIterations/%s/final_auxdata.mat',DateString),'auxdata');
    
    %delete temporary files
    delete(sprintf('SavedIterations/%s/iter_data.mat',DateString));
    delete(sprintf('SavedIterations/%s/plot_data.mat',DateString));
    
    %create figure of final RF pulse
    PlotIterate( objfundata_opt, T_opt, RF_opt, forwardmagn_opt, options, 0, 'final_figure' )
    
    if options.verbosity > 0
        figure, set(gcf,'name','Final RF and gradient waveforms')
        ha = tight_subplot( 3,1 );
        axes(ha(1))
        PlotGradients( auxdata.T, 'pg', 0 );
        axes(ha(2))
        PlotGradients( auxdata.T, 'ps', 0);
        axes(ha(3))
        PlotRF( auxdata.RF, options, 0)
    end

end